/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "XyzE.h"
#include "Serialization.h"

#include "CieXyz.h"


using namespace hxa7241::graphics;




/// commands ---------------------------------------------------------------------------------------
void CieXyz::fromXyzE( const XyzE xyze )
{
	float xyz[3];
	xyze.get( xyz );

	x_m = xyz[0];
	y_m = xyz[1];
	z_m = xyz[2];
}




/// queries ----------------------------------------------------------------------------------------
XyzE CieXyz::toXyzE() const
{
	const float xyz[3] = { x_m, y_m, z_m };
	return XyzE( xyz );
}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& hxa7241::graphics::operator<<( SerializerOut& out, const CieXyz& obj )
{
	out.write( "X", obj.x_m );
	out.write( "Y", obj.y_m );
	out.write( "Z", obj.z_m );

	return out;
}


SerializerIn& hxa7241::graphics::operator>>( SerializerIn& in, CieXyz& obj )
{
	in.read( "X", obj.x_m );
	in.read( "Y", obj.y_m );
	in.read( "Z", obj.z_m );

	return in;
}


const char* CieXyz::CLASS_NAME()
{
	static const char CLASS_NAME[] = "hxa7241::graphics::CieXyz";
	return CLASS_NAME;
}


const char* CieXyz::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "a light measurement in CIE XYZ format";
	return CLASS_DESCRIPTION;
}
