/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ClipCoords_h
#define ClipCoords_h


#include "Primitives.h"
#include "ClipFlags.h"
#include "Vector3f.h"




#include "hxa7241_graphics.h"
namespace hxa7241 { namespace graphics {
	using namespace hxa7241::primitives;


/**
 * stores a points position relative to each of six canonical clipping planes.
 * <br/><br/>
 *
 * assumes subject point is in clip space.<br/>
 * clip space is the unit cube, after a homogeneous perspective transform, but
 * before the perspective divide.<br/><br/>
 *
 * reference:
 * <cite>ch13, 'jim blinns corner, a trip down the graphics pipeline' - blinn
 * (morgan kaufmann 1996)</cite>
 */
class ClipCoords
{
/// standard object services -----------------------------------------------------------------------
public:
	         ClipCoords();
	explicit ClipCoords( const Vector4f& );

	        ~ClipCoords();
	         ClipCoords( const ClipCoords& );
	ClipCoords& operator=( const ClipCoords& );


/// commands ---------------------------------------------------------------------------------------
	        void  setForPoint( const Vector4f& );


/// queries ----------------------------------------------------------------------------------------
	        bool  getInterpolator( const ClipCoords& nextPoint,
	                               float&            interpolator ) const;


/// implementation ---------------------------------------------------------------------------------
protected:
	static  void  copyArray( int length, const float*, float* );


/// fields -----------------------------------------------------------------------------------------
private:
	/// plane:  XLo XHi YLo YHi ZLo ZHi
	/// index:  0   1   2   3   4   5
	float     boundaryCoords_m[6];
	ClipFlags flags_m;
};


}}




#endif//ClipCoords_h
