/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Octree.h"   /// own header is included last


using namespace hxa7241::graphics;




/// test stuff /////////////////////////////////////////////////////////////////////////////////////

/*#include "Vector3f.h"
#include "UnitVector3f.h"
#include "RandomGood.h"
#include "Polar.h"
#include "Exception.h"
#include "Logger.h"
#include <vector>
#include <utility>
#include <set>
#include <map>
#include <time>




class OctreeVisitorTest
	: public OctreeVisitorT<Vector3f>
{
/// standard object services -----------------------------------------------------------------------
public:
	         OctreeVisitorTest();
	         OctreeVisitorTest( std::vector<std::pair<float, const Vector3f*> >* pSelectedItemsBuffer,
	                            const Vector3f& shellCenter,
	                            float           shellRadiusOuter,
	                            float           shellRadiusInner );

	virtual ~OctreeVisitorTest();
private:
	         OctreeVisitorTest( const OctreeVisitorTest& );
	OctreeVisitorTest& operator=( const OctreeVisitorTest& );


/// commands ---------------------------------------------------------------------------------------
protected:
	virtual void  visitItem( const Vector3f& item,
	                         bool            isInside );


/// queries ----------------------------------------------------------------------------------------
	virtual bool  isOverlappingCell( const Vector3f& item,
	                                 const Vector3f& position,
	                                 float           size )                                   const;
	virtual dword getSubcellOverlaps( const Vector3f& item,
	                                  const Vector3f& planePositions )                        const;

public:
	virtual ESelection isSelected( const OctreeBound&,
	                               bool isInside )                                            const;
	virtual dword      getSubcellSelections( const OctreeBound&,
	                                         bool isInside )                                  const;


/// implementation ---------------------------------------------------------------------------------
protected:
	static  bool       isItemOverlappingShell( const Vector3f& item,
	                                           const Vector3f& center,
	                                           float           radiusOuter,
	                                           float           radiusInner );
	static  ESelection isCellOverlappingShell( const OctreeBound& cellBound,
	                                           const Vector3f&    center,
	                                           float              outerRadius,
	                                           float              innerRadius );
	static  dword      getSubcellShellOverlaps( const OctreeBound& cellBound,
	                                            const Vector3f&    center,
	                                            float              outerRadius,
	                                            float              innerRadius );

	static  void  getComponentDistancesSquared( const Vector3f& cellCenter,
	                                            float           cellSize,
	                                            const Vector3f& position,
	                                            Vector3f        dxyzs[2] );
	static  float getDistanceSquared( const Vector3f&,
	                                  const Vector3f& );


/// fields -----------------------------------------------------------------------------------------
private:
	/// external selection buffer
	std::vector<std::pair<float, const Vector3f*> >* pSelectedItems_m;

	/// shell
	Vector3f center_m;
	float    radiusOuter_m;
	float    radiusInner_m;
};




/// standard object services -----------------------------------------------------------------------
OctreeVisitorTest::OctreeVisitorTest()
 :	OctreeVisitorT<Vector3f>(),
	pSelectedItems_m( 0 ),
	center_m        (),
	radiusOuter_m   (),
	radiusInner_m   ()
{
}


OctreeVisitorTest::OctreeVisitorTest
(
	std::vector<std::pair<float, const Vector3f*> >* pSelectedItemsBuffer,
	const Vector3f& center,
	const float     radiusOuter,
	const float     radiusInner
)
 :	OctreeVisitorT<Vector3f>(),
	pSelectedItems_m( pSelectedItemsBuffer ),
	center_m        ( center ),
	radiusOuter_m   ( radiusOuter ),
	radiusInner_m   ( radiusInner )
{
}


OctreeVisitorTest::~OctreeVisitorTest()
{
}




/// commands ---------------------------------------------------------------------------------------
void OctreeVisitorTest::visitItem
(
	const Vector3f& item,
	const bool      isInside
)
{
	/// only act if inclusion condition met
	if( isInside ||
		isItemOverlappingShell( item, center_m, radiusOuter_m, radiusInner_m ) )
	{
		if( pSelectedItems_m != 0 )
		{
			/// get distance of item from shell center
			const float distance2 = getDistanceSquared( item, center_m );

			/// add item and distance to collection
			pSelectedItems_m->push_back( std::pair<float, const Vector3f*>( distance2, &item ) );
			//selectedItems_m.insert( std::multimap<float, const Vector3f*>::value_type( distance2, &item ) );
		}
	}
}




/// queries ----------------------------------------------------------------------------------------
bool OctreeVisitorTest::isOverlappingCell
(
	const Vector3f& item,
	const Vector3f& position,
	const float     size
) const
{
	const Vector3f& itemPosition = item;

	bool isOverlapped;

	const float offsetX = itemPosition.getX() - position.getX();
	isOverlapped  = (offsetX >= 0.0f) & (offsetX < size);

	const float offsetY = itemPosition.getY() - position.getY();
	isOverlapped &= (offsetY >= 0.0f) & (offsetY < size);

	const float offsetZ = itemPosition.getZ() - position.getZ();
	isOverlapped &= (offsetZ >= 0.0f) & (offsetZ < size);


	return isOverlapped;
}


dword OctreeVisitorTest::getSubcellOverlaps
(
	const Vector3f& item,
	const Vector3f& planePositions
) const
{
	const Vector3f& itemPosition = item;

	const bool isHigherX = itemPosition.getX() >= planePositions.getX();
	const bool isHigherY = itemPosition.getY() >= planePositions.getY();
	const bool isHigherZ = itemPosition.getZ() >= planePositions.getZ();

	const dword xBits = (dword(isHigherX) - 1) ^ 0xAA;
	const dword yBits = (dword(isHigherY) - 1) ^ 0xCC;
	const dword zBits = (dword(isHigherZ) - 1) ^ 0xF0;

	return (xBits & yBits & zBits) & 0xFF;
}


OctreeVisitor::ESelection OctreeVisitorTest::isSelected
(
	const OctreeBound& octreeBound,
	const bool         isInside
) const
{
	ESelection s = INSIDE;

	if( !isInside )
	{
		s = isCellOverlappingShell( octreeBound, center_m, radiusOuter_m, radiusInner_m );
	}

	return s;
}


dword OctreeVisitorTest::getSubcellSelections
(
	const OctreeBound& octreeBound,
	const bool         isInside
) const
{
	dword s = ALL_INSIDE;

	if( !isInside )
	{
		s = getSubcellShellOverlaps( octreeBound, center_m, radiusOuter_m, radiusInner_m );
	}

	return s;
}




/// implementation ---------------------------------------------------------------------------------
bool OctreeVisitorTest::isItemOverlappingShell
(
	const Vector3f& item,
	const Vector3f& center,
	const float     radiusOuter,
	const float     radiusInner
)
{
	const float d2 = getDistanceSquared( item, center );

	return (d2 <  (radiusOuter * radiusOuter)) &
	       (d2 >= (radiusInner * radiusInner));
}


OctreeVisitor::ESelection OctreeVisitorTest::isCellOverlappingShell
(
	const OctreeBound& cellBound,
	const Vector3f&    center,
	const float        outerRadius,
	const float        innerRadius
)
{
	/// distance = sqrt( (boundCenter - center).dotSelf() )
	/// out =  distance >= (outerRadius + boundRadius) |
	///        distance <  (innerRadius - boundRadius)
	/// in  =  distance <  (outerRadius - boundRadius) &
	///       (distance >= (innerRadius + boundRadius) | innerRadius == 0 )


	const Vector3f& cellCenter      = cellBound.getCenter();
	const float     cellBoundRadius = cellBound.getRadius();

	const float dx = cellCenter.getX() - center.getX();
	const float dy = cellCenter.getY() - center.getY();
	const float dz = cellCenter.getZ() - center.getZ();

	const float d2 = (dx * dx) + (dy * dy) + (dz * dz);

	const float oPb  = outerRadius + cellBoundRadius;
	const float oPb2 = oPb * oPb;
	const float iMb  = innerRadius - cellBoundRadius;
	const float iMb2 = iMb * iMb;

	const bool isOut = (d2 >= oPb2) | ((d2 < iMb2) & (iMb > 0.0f));

	bool isIn = false;
	if( !isOut )
	{
		const float oMb  = outerRadius - cellBoundRadius;
		const float oMb2 = oMb * oMb;
		const float iPb  = innerRadius + cellBoundRadius;
		const float iPb2 = iPb * iPb;

		isIn = ((d2 < oMb2) & (oMb > 0.0f)) & ((d2 >= iPb2) | (innerRadius <= 0.0f));
	}

	///			in	!out
	/// out		0	0
	/// partial	0	1
	/// in		1	1
	return ESelection( (dword(isIn) << 1) | dword(!isOut) );
}


dword OctreeVisitorTest::getSubcellShellOverlaps
(
	const OctreeBound& cellBound,
	const Vector3f&    center,
	const float        outerRadius,
	const float        innerRadius
)
{
	Vector3f dxyzs[2];
	getComponentDistancesSquared( cellBound.getCenter(), cellBound.getSize(), center, dxyzs );

	const float subcellRadius = cellBound.getRadius() * 0.5f;

	const float opb    = outerRadius + subcellRadius;
	const float opbs   = opb * opb;
	const float imb    = innerRadius - subcellRadius;
	const float imbs   = imb * imb;
	const bool  imblez = (imb <= 0.0f);

	const float omb    = outerRadius - subcellRadius;
	const float ombs   = omb * omb;
	const float ipb    = innerRadius + subcellRadius;
	const float ipbs   = ipb * ipb;
	const bool  ombgz  = (omb > 0.0f);

	dword overlaps = 0;
	for( int i = 8;  i-- > 0; )
	{
		const float ds = dxyzs[i        & 1].getX() +
						 dxyzs[(i >> 1) & 1].getY() +
						 dxyzs[(i >> 2) & 1].getZ();

		const bool isNotOut = ((ds >= imbs) | imblez) &  (ds <  opbs);
		const bool isIn     = ((ds <  ombs) & ombgz ) & ((ds >= ipbs) | (innerRadius <= 0.0f));
		overlaps = (overlaps << 2) | ((dword(isIn) << 1) | dword(isNotOut));
	}

	return overlaps & 0xFFFF;
}


void OctreeVisitorTest::getComponentDistancesSquared
(
	const Vector3f& cellCenter,
	const float     cellSize,
	const Vector3f& position,
	Vector3f        dxyzs[2]
)
{
	const float dcx = cellCenter.getX() - position.getX();
	const float dcy = cellCenter.getY() - position.getY();
	const float dcz = cellCenter.getZ() - position.getZ();

	const float subcellHalfSize = cellSize * 0.25f;

	const float dx[2] = { (dcx - subcellHalfSize), (dcx + subcellHalfSize) };
	const float dy[2] = { (dcy - subcellHalfSize), (dcy + subcellHalfSize) };
	const float dz[2] = { (dcz - subcellHalfSize), (dcz + subcellHalfSize) };

	dxyzs[0].setXYZ( (dx[0] * dx[0]), (dy[0] * dy[0]), (dz[0] * dz[0]) );
	dxyzs[1].setXYZ( (dx[1] * dx[1]), (dy[1] * dy[1]), (dz[1] * dz[1]) );
}


float OctreeVisitorTest::getDistanceSquared
(
	const Vector3f& item,
	const Vector3f& point
)
{
	const Vector3f& itemPosition = item;

	const float dx = itemPosition.getX() - point.getX();
	const float dy = itemPosition.getY() - point.getY();
	const float dz = itemPosition.getZ() - point.getZ();

	return (dx * dx) + (dy * dy) + (dz * dz);
}





void fillItemsAndOctree
(
	bool                          isNoise,
	const bool                    isPlane,
	const Vector3f&               position,
	const float                   size,
	hxa7241::utility::RandomGood& rand,
	std::vector<Vector3f>&        items,
	Octree<Vector3f>&             octree
)
{
	if( !(isNoise | isPlane) )
	{
		isNoise = true;
	}


	/// add items randomly positioned
	if( isNoise )
	{
		OctreeVisitorTest visitor;

		const dword end = isPlane ? items.size()/2 : items.size();
		for( dword c = 0;  c < end;  ++c )
		{
			const float x = rand.getFloat();
			rand.next();
			const float y = rand.getFloat();
			rand.next();
			const float z = rand.getFloat();
			rand.next();

			items[c] = Vector3f( x, y, z );
			(items[c] *= size) += position;
			const bool isInserted = octree.insertItem( items[c], visitor );

			if( !isInserted )
			{
				hxa7241::utility::Logger() << "*** failed to insert item " << c << " " << &(items[c]) << " " <<items[c];
			}
		}
	}


	/// add items planarly positioned
	if( isPlane )
	{
		/// make plane coord frame
		UnitVector3f dir0;
		UnitVector3f dir1;
		{
			float x, y, z;

			const float a = rand.getFloat();
			rand.next();
			const float b = rand.getFloat();
			rand.next();
			hxa7241::graphics::squareToSphere( a, b, &x, &y, &z );
			dir0.setXYZ( x, y, z );

			const float c = rand.getFloat();
			rand.next();
			const float d = rand.getFloat();
			rand.next();
			hxa7241::graphics::squareToSphere( c, d, &x, &y, &z );
			dir1.setXYZ( x, y, z );

			dir1.orthogonalizeEq( dir0 );

			//hxa7241::utility::Logger() << "plane: " << dir0 << " " << dir1;
		}

		OctreeVisitorTest visitor;

		dword c = isNoise ? items.size()/2 : 0;
		for( dword end = items.size();  c < end;  ++c )
		{
			const float x = (rand.getFloat() - 0.5f) * 0.7f;
			rand.next();
			const float y = (rand.getFloat() - 0.5f) * 0.7f;
			rand.next();

			items[c] = (dir0.getVector() * x) + (dir1.getVector() * y);
			((items[c] *= size) += position) += Vector3f::HALF();
			const bool isInserted = octree.insertItem( items[c], visitor );

			if( !isInserted )
			{
				hxa7241::utility::Logger() << "*** failed to insert item " << c << " " << &(items[c]) << " " <<items[c];
			}
		}
	}


	{
		Vector3f min( Vector3f::MAX() );
		Vector3f max( Vector3f::MIN() );
		for( dword i = items.size();  i-- > 0; )
		{
			min.clampToMaxOf( items[i] );
			max.clampToMinOf( items[i] );
		}

		hxa7241::utility::Logger() << "\titems min: " << min << "  items max: " << max;
	}
}




void octreeinfotest
(
	const hxa7241::graphics::Octree<Vector3f>& octree,
	const dword itemsLength,
	const dword depthMax,
	const dword leafSizeMax
)
{
	dword byteSize, leafCount, elementCount, maxDepth;
	octree.getInfo( byteSize, leafCount, elementCount, maxDepth );

	hxa7241::utility::Logger() << "byteSize: " << byteSize << ",  leafCount: " << leafCount << ",  elementCount: " << elementCount << ",  maxDepth: " << maxDepth << ",  isEmpty: " << octree.isEmpty();


	if( (elementCount == 0) != octree.isEmpty() )
	{
		hxa7241::utility::Logger() << "*** element count doesnt match empty check";
	}

	if( elementCount != itemsLength )
	{
		hxa7241::utility::Logger() << "*** item and element counts not matched";
	}

	if( maxDepth > depthMax )
	{
		hxa7241::utility::Logger() << "*** depth limit exceeded";
	}

	if( ((elementCount/leafCount) > leafSizeMax) & (maxDepth < depthMax) )
	{
		hxa7241::utility::Logger() << "*** leaf content size exceeded";
	}
}




void test_Octreeview()
{
	hxa7241::utility::Logger() << "\n=== octree view test ===\n";

	static const dword DEPTH_CLAMP = 16;
	static const dword LEAF_CLAMP  = 2;
	static const Vector3f OCTREE_POS( 0.0f, 0.0f, 0.0f );
	static const float    OCTREE_SIZE( 1.0f );

	hxa7241::graphics::Octree<Vector3f> octree( OCTREE_POS, OCTREE_SIZE, LEAF_CLAMP, DEPTH_CLAMP );
	OctreeVisitorTest visitor;


	const Vector3f ps0[] = { Vector3f(0.1f, 0.1f, 0.1f),
	                         Vector3f(0.4f, 0.1f, 0.1f),
	                         Vector3f(0.1f, 0.6f, 0.1f),
	                         Vector3f(0.1f, 0.4f, 0.1f) };
	const Vector3f ps1[] = { Vector3f(0.1f, 0.1f, 0.1f),
	                         Vector3f(0.6f, 0.1f, 0.1f),
	                         Vector3f(0.1f, 0.6f, 0.1f),
	                         Vector3f(0.6f, 0.6f, 0.1f),
	                         Vector3f(0.1f, 0.1f, 0.6f),
	                         Vector3f(0.6f, 0.1f, 0.6f),
	                         Vector3f(0.1f, 0.6f, 0.6f),
	                         Vector3f(0.6f, 0.6f, 0.6f) };
	const Vector3f ps2[] = { Vector3f(0.1f, 0.1f, 0.1f),
	                         Vector3f(0.1f, 0.1f, 0.1f),
	                         Vector3f(0.1f, 0.1f, 0.1f),
	                         Vector3f(0.1f, 0.1f, 0.1f) };
	const Vector3f ps3[] = { Vector3f::ONE(),
	                         Vector3f::ALMOST_ONE(),
	                         Vector3f::HALF() + Vector3f::EPSILON(),
	                         Vector3f::HALF(),
	                         Vector3f::HALF() - Vector3f::EPSILON(),
	                         Vector3f::EPSILON(),
	                         Vector3f(FLOAT_MIN_POS, FLOAT_MIN_POS, FLOAT_MIN_POS),
	                         Vector3f::ZERO() };

	const Vector3f* pss[] = { ps0, ps1, ps2, ps3 };
	const dword     psl[] = { (sizeof(ps0)/sizeof(*ps0)), (sizeof(ps1)/sizeof(*ps1)), (sizeof(ps2)/sizeof(*ps2)), (sizeof(ps3)/sizeof(*ps3)) };


	hxa7241::utility::Logger() << octree;

	for( int ii = 0;  ii < (sizeof(pss)/sizeof(*pss));  ++ii )
	{
		hxa7241::utility::Logger() << "/// " << ii;

		const Vector3f* ps = pss[ii];

		for( int i = 0;  i < psl[ii];  ++i )
		{
			const bool b = octree.insertItem( ps[i], visitor );
			hxa7241::utility::Logger() << (b ? "inserted " : "*** failed to insert ") << i << " " << &(ps[i]) << " " << ps[i];
			//hxa7241::utility::Logger() << octree;
		}

		octreeinfotest( octree, psl[ii], DEPTH_CLAMP, LEAF_CLAMP );

		if( ii == 0 )
		{
			hxa7241::utility::Logger() << octree;

			hxa7241::graphics::Octree<Vector3f> octree2( octree );
			hxa7241::utility::Logger() << "copy construct ";
			hxa7241::utility::Logger() << octree2;

			hxa7241::graphics::Octree<Vector3f> octree3( Vector3f::ZERO(), 1.0f, LEAF_CLAMP, 4 );
			hxa7241::utility::Logger() << "copy assign ";
			octree3 = octree2;
			hxa7241::utility::Logger() << octree3;
		}

		for( int i = psl[ii];  i-- > 0; )
		{
			const bool b = octree.removeItem( ps[i], visitor );
			hxa7241::utility::Logger() << (b ? "removed " : "*** failed to remove ") << i << " " << &(ps[i]) << " " << ps[i];
			//hxa7241::utility::Logger() << octree;
		}
	}


	{
		std::vector<Vector3f> items( 32 );
		hxa7241::graphics::Octree<Vector3f> octree( OCTREE_POS, OCTREE_SIZE, LEAF_CLAMP, DEPTH_CLAMP );

		const dword seed = dword(std::time(0));
		hxa7241::utility::RandomGood rand( seed );
		hxa7241::utility::Logger() << "seed " << seed;

		fillItemsAndOctree( false, true, OCTREE_POS + Vector3f( 0.001f, 0.001f, 0.001f ), 0.998f, rand, items, octree );

		hxa7241::utility::Logger() << "\tOCTREE_POS: " << OCTREE_POS << "  OCTREE_POS+SIZE: " << (OCTREE_POS + Vector3f(OCTREE_SIZE,OCTREE_SIZE,OCTREE_SIZE));

		octreeinfotest( octree, items.size(), DEPTH_CLAMP, LEAF_CLAMP );
		hxa7241::utility::Logger() << octree;


		for( int i = items.size();  i-- > 0; )
		{
			const bool b = octree.removeItem( items[i], visitor );
			hxa7241::utility::Logger() << (b ? "removed " : "*** failed to remove ") << i << " " << &(items[i]) << " " << items[i];

			if( i == 2 )
			{
				hxa7241::utility::Logger() << octree;
			}
		}
		hxa7241::utility::Logger() << octree;
		hxa7241::utility::Logger() << (octree.isEmpty() ? "isEmpty" : "not isEmpty");
	}
}




void test_Octreeoverlaps()
{
	hxa7241::utility::Logger() << "\n=== octree visitor selection test ===\n";


	static const Vector3f POS( -10.0f, 0.1f, -2.5f );
	static const float    SCL( 3.333f );

	const OctreeBound bound( Vector3f(0.0f, 0.0f, 0.0f) + POS, 1.0f * SCL );

	const Vector3f sc[] = { Vector3f( 0.25f, 0.25f, 0.25f ) * SCL + POS,
	                        Vector3f( 0.75f, 0.25f, 0.25f ) * SCL + POS,
	                        Vector3f( 0.25f, 0.75f, 0.25f ) * SCL + POS,
	                        Vector3f( 0.75f, 0.75f, 0.25f ) * SCL + POS,
	                        Vector3f( 0.25f, 0.25f, 0.75f ) * SCL + POS,
	                        Vector3f( 0.75f, 0.25f, 0.75f ) * SCL + POS,
	                        Vector3f( 0.25f, 0.75f, 0.75f ) * SCL + POS,
	                        Vector3f( 0.75f, 0.75f, 0.75f ) * SCL + POS };

	//const float sr ( 0.934f * SCL );
	const float sro( 0.934f * SCL );
	const float sri( 0.066f * SCL );

	static const char* OVERLAP[] = { "       ", "~~~~~~~", "xxxxxxx", "INSIDE " };


	hxa7241::utility::Logger() << "\t" << bound.getPosition() << " " << bound.getSize();
	for( int i = 0;  i < 8;  ++i )
	{
		OctreeVisitorTest visitor( 0, sc[i], sro, sri );

		const dword overlaps = visitor.getSubcellSelections( bound, false );
		//const dword overlapsSh = bound.getSubcellShellOverlaps( sc[i], sro, sri );

		hxa7241::utility::Logger() << "\t" << i << "  " << sc[i] << " " << sro << " " << sri << "   " << (void*)overlaps;

		for( int j = 0;  j < 8;  ++j )
		{
			const OctreeBound subBound( bound, j );
			const OctreeVisitor::ESelection overlap = visitor.isSelected( subBound, false );
			//const OctreeBound::EOverlap overlap = subBound.isOverlappingShell( sc[i], sro, sri );

			const char*const s1 = OVERLAP[ ((overlaps >> (j << 1)) & 0x03) ];
			const char*const s3 = OVERLAP[ overlap ];
			//const char*const s0 = OVERLAP[ ((overlapsSp >> (j << 1)) & 0x03) ];
			//const char*const s2 = OVERLAP[ overlap0 ];

			hxa7241::utility::Logger() << "\t\t" << j << "    " << s1 << "  " << s3 << "    " << ((s1 != s3) ? "*** unmatching" : "");
		}
	}
}


void test_Octreetemp()
{
//	const Vector3f planePositions( 0.0f, 0.0f, 0.0f );
//
//	const Vector3f items[] = { Vector3f( -1.0f, -1.0f, -1.0f ),
//	                           Vector3f( +1.0f, -1.0f, -1.0f ),
//	                           Vector3f( -1.0f, +1.0f, -1.0f ),
//	                           Vector3f( +1.0f, +1.0f, -1.0f ),
//	                           Vector3f( -1.0f, -1.0f, +1.0f ),
//	                           Vector3f( +1.0f, -1.0f, +1.0f ),
//	                           Vector3f( -1.0f, +1.0f, +1.0f ),
//	                           Vector3f( +1.0f, +1.0f, +1.0f ) };
//
//
//	hxa7241::utility::Logger() << "\tvisitor.getSubcellOverlaps";
//
//	OctreeVisitorTest visitor( (std::set<const void*>*)0 );
//
//	for( int i = 0;  i < 8;  ++i )
//	{
//		const dword overlaps = visitor.getSubcellOverlapsT( (void*)&(items[i]), planePositions );
//
//		hxa7241::utility::Logger() << "\t" << i << "  " << items[i] << "  " << (void*)overlaps;
//	}
}




void hxa7241::graphics::test_Octree()
{
	try
	{
		//test_Octreetemp();

		test_Octreeoverlaps();

		test_Octreeview();

		//test_Octreevisit();
	}
	catch( const std::exception& e )
	{
		hxa7241::utility::Logger() << "*** exception caught: " << e.what();
	}
	catch( ... )
	{
		hxa7241::utility::Exception e;
		hxa7241::utility::Logger() << "*** exception caught: " << e.what();
	}
}*/


void hxa7241::graphics::test_Octree()
{
}
















/*class OctreeVisitorTest
	: public OctreeVisitorT<Vector3f>
{
/// standard object services -----------------------------------------------------------------------
public:
	         OctreeVisitorTest( std::set<const void*>* pSet );
	         OctreeVisitorTest( std::multimap<float, const void*>* pMap );

	virtual ~OctreeVisitorTest();
private:
	         OctreeVisitorTest( const OctreeVisitorTest& );
	OctreeVisitorTest& operator=( const OctreeVisitorTest& );


/// commands ---------------------------------------------------------------------------------------
protected:
	virtual void  visitItem( const Vector3f& item );


/// queries ----------------------------------------------------------------------------------------
	virtual void  getItemBound( const Vector3f& item,
	                            Vector3f&       position,
	                            Vector3f&       size )                                     const;
	virtual void  getItemBound( const Vector3f& item,
	                            Vector3f&       center,
	                            float&          radius )                                   const;

	virtual dword getSubcellOverlaps( const Vector3f& item,
	                                  const Vector3f& planePositions )                     const;
	virtual bool  isOverlappingCube( const Vector3f& item,
	                                 const Vector3f& position,
	                                 float           size )                                const;
	virtual bool  isOverlappingSphere( const Vector3f& item,
	                                   const Vector3f& center,
	                                   float           radius )                            const;
	virtual bool  isOverlappingShell( const Vector3f& item,
	                                  const Vector3f& center,
	                                  float           radiusOuter,
	                                  float           radiusInner )                        const;
	virtual float getDistance( const Vector3f& item,
	                           const Vector3f& position )                                  const;


public:
	        bool  getIsUnmatching() const { return isUnmatching_m; };
	        dword getMatching() const { return matching_m; };


/// fields -----------------------------------------------------------------------------------------
private:
	std::set<const void*>* pSet_m;
	bool isUnmatching_m;

	std::multimap<float, const void*>* pMap_m;
	//std::multimap<float, const void*>::const_iterator mapIt_m;
	dword matching_m;
};




/// standard object services -----------------------------------------------------------------------
OctreeVisitorTest::OctreeVisitorTest
(
	std::set<const void*>* pSet
)
 :	OctreeVisitorT<Vector3f>(),
	pSet_m( pSet ),
	isUnmatching_m( false ),
	pMap_m( 0 ),
	//mapIt_m(),
	matching_m( 0 )
{
}


OctreeVisitorTest::OctreeVisitorTest
(
	std::multimap<float, const void*>* pMap
)
 :	OctreeVisitorT<Vector3f>(),
	pSet_m( 0 ),
	isUnmatching_m( false ),
	pMap_m( pMap ),
	//mapIt_m( pMap->begin() ),
	matching_m( 0 )
{
}


OctreeVisitorTest::~OctreeVisitorTest()
{
}


//OctreeVisitorTest::OctreeVisitorTest
//(
//	const OctreeVisitorTest& //other
//)
// :	OctreeVisitorT<Vector3f>()
//{
//}
//
//
//OctreeVisitorTest& OctreeVisitorTest::operator=
//(
//	const OctreeVisitorTest& other
//)
//{
//
//	return *this;
//}


/// commands ---------------------------------------------------------------------------------------
void OctreeVisitorTest::visitItem
(
	const Vector3f& item
)
{
	if( pSet_m != 0 )
	{
		const bool isErased = bool(pSet_m->erase( &item ));

		if( !isErased )
		{
			hxa7241::utility::Logger() << "\t*** set visiting unfound item: " << &item << " " << item;
			isUnmatching_m = true;
		}
		else
		{
			//hxa7241::utility::Logger() << "\tset visiting item: " << &item << " " << item;
		}
	}

	if( pMap_m != 0 )
	{
		bool found = false;
		{
			std::multimap<float, const void*>::iterator it( pMap_m->begin() );

			if( it != pMap_m->end() )
			{
				const float distance = it->first;

				do
				{
					found = (it->second == &item);
				}
				while( !found && ((++it)->first == distance) );

				if( found )
				{
					pMap_m->erase( it );
				}
			}
		}

		if( !found )
		{
			hxa7241::utility::Logger() << "\t*** map visiting unmatched item: " << &item << " " << item;
			matching_m = -1;
		}
		else
		{
			//hxa7241::utility::Logger() << "\tmap visiting item: " << matching_m << "  " << &item << " " << item;
			matching_m = matching_m != -1 ? matching_m + 1 : -1;
		}


//		if( mapIt_m->second != &item )
//		{
//			hxa7241::utility::Logger() << "\t*** map visiting unmatched item: " << &item << " " << item;
//			matching_m = -1;
//		}
//		else
//		{
//			hxa7241::utility::Logger() << "\tmap visiting item: " << &item << " " << item;
//			matching_m = matching_m != -1 ? matching_m + 1 : -1;
//		}
//
//		++mapIt_m;
	}
}


/// queries ----------------------------------------------------------------------------------------
void OctreeVisitorTest::getItemBound
(
	const Vector3f& item,
	Vector3f&       position,
	Vector3f&       size
) const
{
	position = item;
	size     = Vector3f::ZERO();
}


void OctreeVisitorTest::getItemBound
(
	const Vector3f& item,
	Vector3f&       center,
	float&          radius
) const
{
	center = item;
	radius = 0.0f;
}


dword OctreeVisitorTest::getSubcellOverlaps
(
	const Vector3f& item,
	const Vector3f& planePositions
) const
{
	const Vector3f& itemPosition = item;

	const bool isHigherX = itemPosition.getX() >= planePositions.getX();
	const bool isHigherY = itemPosition.getY() >= planePositions.getY();
	const bool isHigherZ = itemPosition.getZ() >= planePositions.getZ();

	const dword xBits = (dword(isHigherX) - 1) ^ 0xAA;   /// 10101010
	const dword yBits = (dword(isHigherY) - 1) ^ 0xCC;   /// 11001100
	const dword zBits = (dword(isHigherZ) - 1) ^ 0xF0;   /// 11110000

	return (xBits & yBits & zBits) & 0xFF;


	// dword overlap = 0;
	//
	// overlap = (overlap << 1) | (!isLowerX & !isLowerY & !isLowerZ);
	// overlap = (overlap << 1) | ( isLowerX & !isLowerY & !isLowerZ);
	// overlap = (overlap << 1) | (!isLowerX &  isLowerY & !isLowerZ);
	// overlap = (overlap << 1) | ( isLowerX &  isLowerY & !isLowerZ);
	// overlap = (overlap << 1) | (!isLowerX & !isLowerY &  isLowerZ);
	// overlap = (overlap << 1) | ( isLowerX & !isLowerY &  isLowerZ);
	// overlap = (overlap << 1) | (!isLowerX &  isLowerY &  isLowerZ);
	// overlap = (overlap << 1) | ( isLowerX &  isLowerY &  isLowerZ);
}


bool OctreeVisitorTest::isOverlappingCube
(
	const Vector3f& item,
	const Vector3f& position,
	const float     size
) const
{
	const Vector3f& itemPosition = item;

	bool isOverlapped;

	const float offsetX = itemPosition.getX() - position.getX();
	isOverlapped  = (offsetX >= 0.0f) & (offsetX < size);

	const float offsetY = itemPosition.getY() - position.getY();
	isOverlapped &= (offsetY >= 0.0f) & (offsetY < size);

	const float offsetZ = itemPosition.getZ() - position.getZ();
	isOverlapped &= (offsetZ >= 0.0f) & (offsetZ < size);


	return isOverlapped;
}


bool OctreeVisitorTest::isOverlappingSphere
(
	const Vector3f& item,
	const Vector3f& center,
	const float     radius
) const
{
	const Vector3f& itemPosition = item;

	const float dx = itemPosition.getX() - center.getX();
	const float dy = itemPosition.getY() - center.getY();
	const float dz = itemPosition.getZ() - center.getZ();

	const float d2 = (dx * dx) + (dy * dy) + (dz * dz);

	return d2 < (radius * radius);
}


bool OctreeVisitorTest::isOverlappingShell
(
	const Vector3f& item,
	const Vector3f& center,
	const float     radiusOuter,
	const float     radiusInner
) const
{
	const Vector3f& itemPosition = item;

	const float dx = itemPosition.getX() - center.getX();
	const float dy = itemPosition.getY() - center.getY();
	const float dz = itemPosition.getZ() - center.getZ();

	const float d2 = (dx * dx) + (dy * dy) + (dz * dz);

	return (d2 <  (radiusOuter * radiusOuter)) &
	       (d2 >= (radiusInner * radiusInner));
}


float OctreeVisitorTest::getDistance
(
	const Vector3f& item,
	const Vector3f& position
) const
{
	return item.distance( position );
}*/




/*void test_Octreevisit()
{
	hxa7241::utility::Logger() << "\n=== octree visit test ===\n";


	const float    OCTREE_SIZE = 1.1f;
	const Vector3f OCTREE_POS(-OCTREE_SIZE*0.5f, -OCTREE_SIZE*0.5f, -OCTREE_SIZE*0.5f);
	const dword    LEAF_CLAMP  = 4;
	const dword    DEPTH_CLAMP = 16;

	const float    SPHERE_SIZE = 1.26f * OCTREE_SIZE;
	const Vector3f SPHERE_POS(-SPHERE_SIZE*0.5f, -SPHERE_SIZE*0.5f, -SPHERE_SIZE*0.5f);
	const float    SPHERE_RAD = 1.957f * OCTREE_SIZE;

	const dword    TOTAL_TESTS = 100;//10;//100;
	const dword    QUERY_TESTS = 16;//4;//16;
	const dword    ITEM_COUNT  = 1000;//100;//1000;

	bool isFailedA = false;
	bool isFailedB = false;
	bool isFailedC = false;

	const dword seed = dword(std::time(0));//1046276917;//1045666691;//1045652747;
	hxa7241::utility::RandomGood rand( seed );
	hxa7241::utility::Logger() << "seed " << seed;


	for( int j = 0;  j < TOTAL_TESTS;  ++j )
	{
		/// make items and octree
		std::vector<Vector3f> items( ITEM_COUNT );
		Octree<Vector3f>      octree( OCTREE_POS, OCTREE_SIZE, LEAF_CLAMP, DEPTH_CLAMP);


		/// fill items and octree
		//fillItemsAndOctree( true, true, -Vector3f::HALF(), 1.0f, rand, items, octree );
		{
			/// add items randomly positioned in unit cube
			{
				OctreeVisitorTest visitor( (std::set<const void*>*)0 );
				for( dword c = 0, end = items.size()/2;  c < end;  ++c )
				//for( dword c = 0, end = items.size();  c < end;  ++c )
				{
					const float x = rand.getFloat() - 0.5f;
					rand.next();
					const float y = rand.getFloat() - 0.5f;
					rand.next();
					const float z = rand.getFloat() - 0.5f;
					rand.next();

					items[c] = Vector3f( x, y, z );
					const bool isInserted = octree.insertItem( items[c], visitor );

					if( !isInserted ) hxa7241::utility::Logger() << "*** failed to insert item " << c << " " << &(items[c]) << " " <<items[c];
					//else hxa7241::utility::Logger() << "item " << c << " " << &(items[c]) << " " <<items[c];
				}
			}

			/// add items planarly positioned
			{
				/// make plane coord frame
				UnitVector3f dir0;
				UnitVector3f dir1;
				{
					const float a = rand.getFloat();
					rand.next();
					const float b = rand.getFloat();
					rand.next();
					float x, y, z;
					hxa7241::graphics::squareToSphere( a, b, &x, &y, &z );
					dir0.setXYZ( x, y, z );

					const float c = rand.getFloat();
					rand.next();
					const float d = rand.getFloat();
					rand.next();
					hxa7241::graphics::squareToSphere( c, d, &x, &y, &z );
					dir1.setXYZ( x, y, z );

					dir1.orthogonalizeEq( dir0 );

					//hxa7241::utility::Logger() << "plane: " << dir0 << " " << dir1;
				}

				OctreeVisitorTest visitor( (std::set<const void*>*)0 );
				for( dword c = items.size()/2, end = items.size();  c < end;  ++c )
				//for( dword c = 0, end = items.size();  c < end;  ++c )
				{
					const float x = (rand.getFloat() - 0.5f) * 0.7f;
					rand.next();
					const float y = (rand.getFloat() - 0.5f) * 0.7f;
					rand.next();

					items[c] = (dir0.getVector() * x) + (dir1.getVector() * y);
					const bool isInserted = octree.insertItem( items[c], visitor );

					if( !isInserted ) hxa7241::utility::Logger() << "*** failed to insert item " << c << " " << &(items[c]) << " " <<items[c];
					//else hxa7241::utility::Logger() << "item " << c << " " << &(items[c]) << " " <<items[c];
				}
			}

			//hxa7241::utility::Logger() << octree;
		}


		/// query items and octree
		{
			/// do various queries
			for( int i = 0;  i < QUERY_TESTS;  ++i )
			{
				/// make a query point
				const float x = rand.getFloat();
				rand.next();
				const float y = rand.getFloat();
				rand.next();
				const float z = rand.getFloat();
				rand.next();
				const float r = rand.getFloat();
				rand.next();
				const float n = rand.getFloat();
				rand.next();

				const Vector3f center( (Vector3f(x, y, z) *= SPHERE_SIZE) += SPHERE_POS );
				const float    radius( r * SPHERE_RAD );
				const dword    number( n * ITEM_COUNT );


				//if( (j == 40) & (i == 0) )
				{
					hxa7241::utility::Logger() << "## " << j << " " << i;
					{
						hxa7241::utility::Logger() << "octree: " << octree.getPosition() << " " << octree.getSize() << " " << (octree.getPosition() + (Vector3f::ONE() * octree.getSize()));
						octreeinfotest( octree, items.size(), DEPTH_CLAMP, LEAF_CLAMP );
					}
					hxa7241::utility::Logger() << "query point: " << center << " " << radius << " " << number;


					/// exhaustively query items
					std::set<const void*>             itemsInsideSphere;
					std::multimap<float, const void*> itemsNearest;
					{
						Vector3f min( Vector3f::MAX() );
						Vector3f max( Vector3f::MIN() );
						for( dword ii = items.size();  ii-- > 0; )
						{
							const float distance = items[ii].distance( center );
							if( distance <= radius )
							{
								itemsInsideSphere.insert( &(items[ii]) );
							}

							itemsNearest.insert( std::pair<float, const void*>( distance, &(items[ii]) ) );

							min.clampToMaxOf( items[ii] );
							max.clampToMinOf( items[ii] );
						}
						hxa7241::utility::Logger() << "items min max: " << min << " " << max << "  " << itemsInsideSphere.size() << " " << itemsNearest.size() << " " << items.size();
					}


					/// query octree, sphere
					{
						const dword n = itemsInsideSphere.size();

						OctreeVisitorTest visitor( &itemsInsideSphere );
						octree.visitItemsInsideSphere( center, radius, visitor );

						if( itemsInsideSphere.empty() )
						{
							hxa7241::utility::Logger() << "- sphere all items visited - " << n;
						}
						else
						{
							hxa7241::utility::Logger() << "*** sphere items unvisited: " << itemsInsideSphere.size();
							isFailedA = true;
						}

						if( !visitor.getIsUnmatching() )
						{
							hxa7241::utility::Logger() << "- sphere all visited are items - " << n;
						}
						else
						{
							hxa7241::utility::Logger() << "*** sphere visited non-items";
							isFailedB = true;
						}
					}

					/// query octree, nearest
					{
						{
							std::multimap<float, const void*>::const_iterator it( itemsNearest.begin() );
							std::multimap<float, const void*>::const_iterator end( itemsNearest.end() );
							for( dword n = 0;  it != end;  ++it, ++n )
							{
								//hxa7241::utility::Logger() << "n: " << n << " " << it->first << " " << it->second;
							}
						}

						bool isOutside;// = false;
						{
							const Vector3f lo( octree.getPosition() );
							const Vector3f hi( lo + Vector3f( octree.getSize(), octree.getSize(), octree.getSize() ) );
							const Vector3f posMlo( center - lo );
							const Vector3f posMhi( center - hi );

							isOutside = ((posMlo.getX() <  0.0f) | (posMlo.getY() <  0.0f) | (posMlo.getZ() <  0.0f)) ||
										((posMhi.getX() >= 0.0f) | (posMhi.getY() >= 0.0f) | (posMhi.getZ() >= 0.0f));

							hxa7241::utility::Logger() << "isOutside: " << isOutside;
						}


						OctreeVisitorTest visitor( &itemsNearest );
						octree.visitItemsNearestToPoint( center, number, visitor );

						if( (visitor.getMatching() == 0) & isOutside )
						{
							hxa7241::utility::Logger() << "- nearest position outside -";
						}
						else if( (visitor.getMatching() == number) && (dword(itemsNearest.size()) == (ITEM_COUNT - number)) )
						{
							hxa7241::utility::Logger() << "- nearest all visited items are matched -";
						}
						else
						{
							if( visitor.getMatching() != -1 )
							{
								hxa7241::utility::Logger() << "*** nearest visited unmatched items: " << visitor.getMatching() << "  " << itemsNearest.size();
							}
							else
							{
								hxa7241::utility::Logger() << "*** nearest visited unmatched items";
							}
							isFailedC = true;
						}
					}
				}
			}
		}
	}


	if( !(isFailedA | isFailedB | isFailedC) )
	{
		hxa7241::utility::Logger() << "\n- successful -\n";
	}
	else
	{
		if( isFailedA )
		{
			hxa7241::utility::Logger() << "*** failure sphere: items unvisited ";
		}
		else if( isFailedB )
		{
			hxa7241::utility::Logger() << "*** failure sphere: visited non-items";
		}
		else
		{
			hxa7241::utility::Logger() << "*** failure nearest: visited unmatched items";
		}
	}
}*/
