/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Vector3f.h"
#include "Quaternion.h"
#include "Polar.h"
#include "Serialization.h"

#include "Transplace.h"   /// own header is included last


using namespace hxa7241::graphics;




/// standard object services -----------------------------------------------------------------------
Transplace::Transplace()
 :	Transform(),
	matrix_m ( Matrix3f::IDENTITY() )
	//scaling_m( 1.0f )
{
}


Transplace::Transplace
(
	//const float     scale,
	const Vector3f& rotation,
	const Vector3f& position
)
 :	Transform(),
	matrix_m()
	//scaling_m()
{
	//Transplace::setTransplace( scale, rotation, position );
	Transplace::setTransplace( rotation, position );
}


Transplace::Transplace
(
	const Transplace& other,
	const bool        invert
)
 :	Transform( other ),
	matrix_m ( other.matrix_m )
	//scaling_m( other.scaling_m )
{
	if( invert )
	{
		Transplace::invert();
	}
}


Transplace::~Transplace()
{
}


Transplace::Transplace( const Transplace& other )
 :	Transform( other ),
	matrix_m ( other.matrix_m )
	//scaling_m( other.scaling_m )
{
}


Transplace& Transplace::operator=( const Transplace& other )
{
	assign( other );

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
void Transplace::setTransplace
(
	//const float     scale,
	const Vector3f& rotation,
	const Vector3f& position
)
{
//	Matrix3f m( Matrix3f::IDENTITY() );
//
//	//matrix_m.setToScale( Vector3f( scale, scale, scale ) );
//
//	//m.setToRotateX( degreesToRadians(rotation.getX()) );
//	//matrix_m.multiplyEq( m );
//	matrix_m.setToRotateX( degreesToRadians(rotation.getX()) );
//
//	m.setToRotateY( degreesToRadians(rotation.getY()) );
//	matrix_m.multiplyEq( m );
//
//	m.setToRotateZ( degreesToRadians(rotation.getZ()) );
//	matrix_m.multiplyEq( m );
//
//	m.setToTranslate( position );
//	matrix_m.multiplyEq( m );
//
//
//	//scaling_m = scale;


	matrix_m.setToRotateXYZ( degreesToRadians(rotation.getX()), degreesToRadians(rotation.getY()), degreesToRadians(rotation.getZ()) );

	Matrix3f m;
	m.setToTranslate( position );
	matrix_m.multiplyEq( m );
}


void Transplace::setTransplace
(
	const Quaternion& rotation,
	const Vector3f&   position
)
{
	rotation.getMatrix( matrix_m );

	Matrix3f m;
	m.setToTranslate( position );
	matrix_m.multiplyEq( m );
}


void Transplace::setTransplace
(
	const Transplace& other,
	const bool        invert
)
{
	assign( other );

	if( invert )
	{
		this->invert();
	}
}


Transplace& Transplace::invert()
{
	matrix_m.invert();

	//scaling_m = scaling_m != 0.0f ? 1.0f / scaling_m : 0.0f;

	return *this;
}


Transplace& Transplace::operator+=( const Transplace& other )
{
	matrix_m.multiplyEq( other.matrix_m );
	//invMatrix_m = Matrix3f( other.invMatrix_m ).multiplyEq( invMatrix_m );

	//scaling_m *= other.scaling_m;

	return *this;
}




/// queries ----------------------------------------------------------------------------------------
//float Transplace::getScale() const
//{
//	return scaling_m;
//}


const Vector3f& Transplace::getRotation() const
{
	/// ummm, not done...
	return Vector3f::ZERO();
}


const Vector3f& Transplace::getPosition() const
{
	return matrix_m.getCol3();
}


const Matrix3f& Transplace::getMatrix() const
{
	return matrix_m;
}


void Transplace::transform
(
	Vector3f& point
) const
{
   matrix_m.multiply( point );
}


void Transplace::transform
(
	UnitVector3f& uvect
) const
{
   matrix_m.multiply( uvect );
}


void Transplace::transform
(
	float& //magnitude
) const
{
	//magnitude *= scaling_m;
}




/// implementation ---------------------------------------------------------------------------------
void Transplace::assign
(
	const Transplace& other
)
{
	if( &other != this )
	{
		Transform::operator=( other );

		matrix_m  = other.matrix_m;
		//scaling_m = other.scaling_m;
	}
}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& hxa7241::graphics::operator<<( SerializerOut& out, const Transplace& obj )
{
	//out.write( "scale",    obj.getScale()    );
	out.write( "rotation", obj.getRotation() );
	out.write( "position", obj.getPosition() );

	return out;
}


SerializerIn& hxa7241::graphics::operator>>( SerializerIn& in, Transplace& obj )
{
	//float    scale;
	Vector3f rotation;
	Vector3f position;
	//in.read( "scale",    scale    );
	in.read( "rotation", rotation );
	in.read( "position", position );

	//obj.setTransplace( scale, rotation, position );
	obj.setTransplace( rotation, position );

	return in;
}


const char* Transplace::CLASS_NAME()
{
	static const char CLASS_NAME[] = "hxa7241::graphics::Transplace";
	return CLASS_NAME;
}


const char* Transplace::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "a rigid transform: translation, rotation";
	return CLASS_DESCRIPTION;
}
