/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Render_h
#define Render_h


#include "Primitives.h"

#include "perceptuum2_modelling.h"
#include "perceptuum2_imaging.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Scene;
	using perceptuum2::imaging::ImageCieXyz;


class Render
{
/// standard object services -----------------------------------------------------------------------
protected:
	             Render();
public:
	virtual     ~Render();
private:
	             Render( const Render& );
	     Render& operator=( const Render& );


/// main command -----------------------------------------------------------------------------------
public:
	        void render( const Scene&            scene,
	                     const RenderingOptions& options,
	                     RenderingStatus&        status,
	                     ImageCieXyz&            image,
	                     RenderingState&         state );


/// other commands ---------------------------------------------------------------------------------
public:
	        void processImage( const void*  options,
	                           ImageCieXyz& image ) const;


/// overrideables ----------------------------------------------------------------------------------
protected:
	virtual bool shouldGo()          const;
	virtual bool shouldStop()        const;
	virtual bool shouldStopAtFrame() const;

	virtual void attendToLineDone( dword lineNumber );
	virtual void attendToFrameDone( bool shouldRepaint );
	virtual void attendToRenderDone( bool shouldRepaint );


/// fields -----------------------------------------------------------------------------------------
private:
	static const char RENDER_STARTED[];
	static const char RENDER_DONE_FRAME[];
	static const char RENDER_COMPLETED[];
	static const char RENDER_INTERRUPTED[];
	static const char AT_FRAME_END[];
	static const char AT_LINE_END[];
	static const char AT[];
	static const char DURATION[];
};


}}




#endif//Render_h
