/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderData_h
#define RenderData_h


#include "Primitives.h"

#include "perceptuum2_modelling.h"
#include "perceptuum2_imaging.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Scene;
	using perceptuum2::imaging::ImageCieXyz;


class RenderData
{
/// standard object services -----------------------------------------------------------------------
protected:
	             RenderData();
public:
	virtual     ~RenderData();
private:
	             RenderData( const RenderData& );
	RenderData&  operator=( const RenderData& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual Scene&            getScene()   const;
	virtual RenderingOptions& getOptions() const;
	virtual RenderingStatus&  getStatus()  const;
	virtual RenderingState&   getState()   const;
	virtual ImageCieXyz&      getImage()   const;


/// commands ---------------------------------------------------------------------------------------
protected:
	virtual void setScene  ( Scene*            scene );
	virtual void setOptions( RenderingOptions* options );
	virtual void setStatus ( RenderingStatus*  status );
	virtual void setState  ( RenderingState*   state );
	virtual void setImage  ( ImageCieXyz*      image );


/// fields -----------------------------------------------------------------------------------------
private:
	Scene*            pScene_m;
	RenderingOptions* pOptions_m;
	RenderingStatus*  pStatus_m;
	RenderingState*   pState_m;
	ImageCieXyz*      pImage_m;
};


}}




#endif//RenderData_h
