/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderKernel_h
#define RenderKernel_h


#include "Primitives.h"
#include "SpacetimeIndex.h"
#include "Illumination.h"
#include "RasterizerTracing.h"

#include "perceptuum2_transport.h"
#include "perceptuum2_modelling.h"
#include "perceptuum2_imaging.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::transport::SamplingControl;
	using perceptuum2::transport::SpacetimeIndex;
	using perceptuum2::transport::Illumination;
	using perceptuum2::transport::RasterizerTracing;
	using perceptuum2::imaging::ImageAccumulation;
	using perceptuum2::modelling::Scene;


class RenderKernel
{
/// standard object services -----------------------------------------------------------------------
public:
	         RenderKernel( const Scene&,
	                       const RenderingOptions&,
	                       RenderingState& );

	virtual ~RenderKernel();
private:
	         RenderKernel( const RenderKernel& );
	RenderKernel& operator=( const RenderKernel& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  renderFrame();


/// queries ----------------------------------------------------------------------------------------
public:
	virtual bool  isCompleted()       const;
	virtual float getCompletedRatio() const;
	virtual dword getFrameCount()     const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// refs to persistent objects
	ImageAccumulation& accumulation_m;
	SamplingControl&   samplingControl_m;

	/// instance objects
	SpacetimeIndex     sceneIndex_m;
	Illumination       illumination_m;
	RasterizerTracing  rasterizer_m;
};


}}




#endif//RenderKernel_h
