/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderingOptionsImmutable_h
#define RenderingOptionsImmutable_h


#include "Primitives.h"
#include "Projection.h"
#include "CieXyz.h"
#include <string>

#include "hxa7241_utility.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::transport::Projection;
	using hxa7241::graphics::CieXyz;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;


class RenderingOptionsImmutable
{
/// standard object services -----------------------------------------------------------------------
public:
	                  RenderingOptionsImmutable();

	                 ~RenderingOptionsImmutable();

	                  RenderingOptionsImmutable( const RenderingOptionsImmutable& );
	RenderingOptionsImmutable& operator=( const RenderingOptionsImmutable& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void    setToDefault();
	        void    setImageSize( dword width, dword height );

/// queries ----------------------------------------------------------------------------------------
public:
	        void    getSummary( string& )            const;

	        const Projection& getProjection()        const;
	        dword   getImageWidth()                  const;
	        dword   getImageHeight()                 const;
	        CieXyz& getDefaultObjectColor( CieXyz& ) const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const RenderingOptionsImmutable& );
	friend  SerializerIn&  operator>>( SerializerIn&,        RenderingOptionsImmutable& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
private:
	        void  enforceConstraints();


/// fields -----------------------------------------------------------------------------------------
private:
	Projection projection_m;
	dword      imageWidth_m;
	dword      imageHeight_m;
	CieXyz     defaultObjectColor_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const RenderingOptionsImmutable& );
SerializerIn&  operator>>( SerializerIn&,        RenderingOptionsImmutable& );


}}




#endif//RenderingOptionsImmutable_h
