/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderingState_h
#define RenderingState_h


#include "Primitives.h"
#include "RenderingOptionsImmutable.h"
#include "SamplingControl.h"
#include "ImageAccumulation.h"
#include <string>

#include "hxa7241_utility.h"
#include "perceptuum2_imaging.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::transport::SamplingControl;
	using perceptuum2::imaging::ImageCieXyz;
	using perceptuum2::imaging::ImageAccumulation;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;


class RenderingState
{
/// standard object services -----------------------------------------------------------------------
public:
	         RenderingState();

	virtual ~RenderingState();
	         RenderingState( const RenderingState& );
	RenderingState& operator=( const RenderingState& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setOptions( const RenderingOptionsImmutable& );
	virtual void  setImage( ImageCieXyz* );

	virtual ImageAccumulation& getImageAccumulation();
	virtual SamplingControl&   getSamplingControl();


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void  getSummary( string& )                            const;

	virtual const RenderingOptionsImmutable& getImmutableOptions() const;
	virtual dword getImageWidth()                                  const;
	virtual dword getImageHeight()                                 const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const RenderingState& );
	friend  SerializerIn&  operator>>( SerializerIn&,        RenderingState& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const RenderingState& );


/// fields -----------------------------------------------------------------------------------------
private:
	RenderingOptionsImmutable options_m;
	ImageAccumulation         imageAccumulation_m;
	SamplingControl           samplingControl_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const RenderingState& );
SerializerIn&  operator>>( SerializerIn&,        RenderingState& );


}}




#endif//RenderingState_h
