/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderingStatus_h
#define RenderingStatus_h


#include "Primitives.h"
#include "Float01c.h"
#include <string>

#include "hxa7241_utility.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Float01c;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;


class RenderingStatus
{
/// standard object services -----------------------------------------------------------------------
public:
	         RenderingStatus();

	virtual ~RenderingStatus();
	         RenderingStatus( const RenderingStatus& );
	RenderingStatus& operator=( const RenderingStatus& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  start();
	virtual void  stop();
	virtual void  setFinished();

	virtual void  setCompletedFrames( dword );
	virtual void  setCompletedProportion( float );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void  getSummary( string& )                                      const;
	virtual void  getSummaryLine( string& )                                  const;

	virtual dword getFramesCompleted()                                       const;
	virtual dword getDuration()                                              const;

	virtual void  getEstimatedDurations( dword& complete, dword& remaining ) const;
	virtual float getMeanFrameTime()                                         const;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const RenderingStatus& );
	friend  SerializerIn&  operator>>( SerializerIn&,        RenderingStatus& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const RenderingStatus& );


/// fields -----------------------------------------------------------------------------------------
private:
	bool  isWorking_m;
	bool  isFinished_m;

	dword    workDurationTotal_m;
	dword    framesCompleted_m;
	Float01c fractionCompleted_m;

	dword sessionStartTime_m;
	dword workDurationSession_m;

	static const char STATUS_LINE_START[];
	static const char STATUS_LINE_START2[];
	static const char STATUS_LINE_FRAME[];
	static const char STATUS_LINE_LINE[];
	static const char STATUS_LINE_FRAMECOUNT[];
	static const char STATUS_LINE_IN[];
	static const char STATUS_LINE_DURATION[];
	static const char STATUS_LINE_SECONDS[];
	static const char STATUS_LINE_FINISHED[];
	static const char STATUS_LINE_UNFINISHED[];
	static const char STATUS_LINE_STOPPED[];
	static const char STATUS_LINE_WORKING[];
	static const char STATUS_LINE_FRAMETIME[];
	static const char STATUS_LINE_ESTIMATED_TIME_TOTAL[];
	static const char STATUS_LINE_ESTIMATED_TIME_LEFT[];
	static const char STATUS_LINE_REMAINING[];
	static const char STATUS_LINE_PERCENT_DONE[];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const RenderingStatus& );
SerializerIn&  operator>>( SerializerIn&,        RenderingStatus& );


}}




#endif//RenderingStatus_h
