/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ColorTransformXyzToRgbFp_h
#define ColorTransformXyzToRgbFp_h


#include "Primitives.h"
#include "Vector3f.h"
#include "Matrix3f.h"
#include "Array.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Matrix3f;
	using hxa7241::utility::Array;


class ColorTransformXyzToRgbFp
{
/// standard object services -----------------------------------------------------------------------
public:
	         ColorTransformXyzToRgbFp();
	         ColorTransformXyzToRgbFp( const Vector3f chromaticitiesXyz[3], const Vector3f& whitePointXYZ );

	        ~ColorTransformXyzToRgbFp();
	         ColorTransformXyzToRgbFp( const ColorTransformXyzToRgbFp& );
	ColorTransformXyzToRgbFp& operator=( const ColorTransformXyzToRgbFp& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setSRGB();
	        void  set( const Vector3f chromaticitiesXyz[3], const Vector3f& whitePointXYZ );


/// queries ----------------------------------------------------------------------------------------
public:
	        void  transform( Vector3f& point )                  const;
	        void  transform( const Array<Vector3f>& points )    const;

	        void  invTransform( Vector3f& point )               const;
	        void  invTransform( const Array<Vector3f>& points ) const;


/// fields -----------------------------------------------------------------------------------------
private:
   Matrix3f XYZtoRGBmatrix_m;
   Matrix3f RGBtoXYZmatrix_m;

   static const Vector3f sRGB_CHROMATICITIES[ 3 ];
   static const Vector3f sRGB_WHITEPOINT;
};


}}




#endif//ColorTransformXyzToRgbFp_h
