/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef DisplayMeasures_h
#define DisplayMeasures_h


#include "Primitives.h"
#include "CieXyz.h"
#include "Vector3f.h"

#include "hxa7241_utility.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::Vector3f;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


class DisplayMeasures
{
///// standard object services -----------------------------------------------------------------------
public:
	         DisplayMeasures();

	        ~DisplayMeasures();
	         DisplayMeasures( const DisplayMeasures& );
	DisplayMeasures& operator=( const DisplayMeasures& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setToDefault();


/// queries ----------------------------------------------------------------------------------------
public:
	        void  getBlackPoint( CieXyz& )            const;
	        void  getWhitePoint( CieXyz& )            const;
	        void  getChromaticitiesXyzX( Vector3f& )  const;
	        void  getChromaticitiesXyzY( Vector3f& )  const;
	        void  getChromaticitiesXyzZ( Vector3f& )  const;
	        void  getChromaticitiesXyz( Vector3f[3] ) const;
	        void  getGammaXyz( Vector3f& )            const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const DisplayMeasures& );
	friend  SerializerIn&  operator>>( SerializerIn&,        DisplayMeasures& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
private:
	        void  enforceConstraints();


/// fields -----------------------------------------------------------------------------------------
private:
	CieXyz    blackPoint_m;
	CieXyz    whitePoint_m;
	Vector3f  chromaticitiesXyzX_m;
	Vector3f  chromaticitiesXyzY_m;
	Vector3f  chromaticitiesXyzZ_m;
	Vector3f  gammaXyz_m;

	static const CieXyz   BLACKPOINT_DEFAULT;
	static const CieXyz   WHITEPOINT_DEFAULT;
	static const Vector3f CHROMATICITIES_DEFAULT[3];
	static const Vector3f GAMMA_DEFAULT;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const DisplayMeasures& );
SerializerIn&  operator>>( SerializerIn&,        DisplayMeasures& );


}}




#endif//DisplayMeasures_h
