/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "PngReader.h"

#include "ImageReader.h"


using namespace perceptuum2::imaging;




/// statics
PngReader ImageReader::pngReader_s;

const ImageReader* ImageReader::readers_s[]   = { &pngReader_s };
const dword        ImageReader::noOfReaders_s = sizeof(readers_s) / sizeof(*readers_s);




/// static queries ---------------------------------------------------------------------------------
void ImageReader::getFormatNames
(
	vector<string>& names
)
{
	if( names.size() != udword(noOfReaders_s) )
	{
		names.resize( noOfReaders_s );
	}

	for( int i = 0;  i < noOfReaders_s;  ++i )
	{
		readers_s[i]->getName( names[i] );
	}
}


bool ImageReader::readStatic
(
	istream&     imageBytes,
	const float  adaptLuminance,
	ImageCieXyz& imageXyz
)
{
	bool didRead = false;

	/// try each reader until one recognizes the file
	for( int i = 0;  (i < noOfReaders_s) & !didRead;  ++i )
	{
		didRead = readers_s[i]->read( imageBytes, adaptLuminance, imageXyz );
	}

	return didRead;
}


bool ImageReader::readStatic
(
	istream&        imageBytes,
	ImageRgba&      imageRgba,
	ImageConverter& converter
)
{
	bool didRead = false;

	/// try each reader until one recognizes the file
	for( int i = 0;  (i < noOfReaders_s) & !didRead;  ++i )
	{
		didRead = readers_s[i]->read( imageBytes, imageRgba, converter );
	}

	return didRead;
}


bool ImageReader::readStatic
(
	istream&                          imageBytes,
	Sheet<hxa7241::primitives::byte>& image8Bit
)
{
	bool didRead = false;

	/// try each reader until one recognizes the file
	for( int i = 0;  (i < noOfReaders_s) & !didRead;  ++i )
	{
		didRead = readers_s[i]->read( imageBytes, image8Bit );
	}

	return didRead;
}
