/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageReader_h
#define ImageReader_h


#include "Primitives.h"
#include "Sheet.h"
#include <vector>
#include <string>
#include <iosfwd>

#include "hxa7241_utility.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Sheet;
	using std::vector;
	using std::string;
	using std::istream;


class ImageReader
{
/// standard object services -----------------------------------------------------------------------
protected:
	         ImageReader() {}
public:
	virtual ~ImageReader() {}
private:
	         ImageReader( const ImageReader& );
	ImageReader& operator=( const ImageReader& );


/// static queries ---------------------------------------------------------------------------------
public:
	static  void getFormatNames( vector<string>& names );
	/// returns whether the stream was recognized (and therefore also read)
	static  bool readStatic( istream& imageBytes, float adaptLuminance, ImageCieXyz& );
	static  bool readStatic( istream& imageBytes, ImageRgba&, ImageConverter& );
	static  bool readStatic( istream& imageBytes, Sheet<byte>&  );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getName( string& )                                                        const =0;
	/// returns whether the stream was recognized (and therefore also read)
	virtual bool read( istream& imageBytes, float adaptLuminance, ImageCieXyz& )           const =0;
	virtual bool read( istream& imageBytes, ImageRgba&, ImageConverter& )                  const =0;
	virtual bool read( istream& imageBytes, Sheet<byte>& )                                 const =0;


/// fields -----------------------------------------------------------------------------------------
private:
	static PngReader pngReader_s;

	static const ImageReader* readers_s[];
	static const dword        noOfReaders_s;
};


}}




#endif//ImageReader_h
