/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageWriter_h
#define ImageWriter_h


#include "Primitives.h"
#include <vector>
#include <string>
#include <iosfwd>




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using perceptuum2::imaging::ImageCieXyz;
	using perceptuum2::imaging::ImageConverter;
	using perceptuum2::imaging::PngWriter;
	using perceptuum2::imaging::RealPngWriter;
	using std::vector;
	using std::string;
	using std::ostream;


class ImageWriter
{
/// standard object services -----------------------------------------------------------------------
protected:
	         ImageWriter() {}
public:
	virtual ~ImageWriter() {}
private:
	         ImageWriter( const ImageWriter& );
	ImageWriter& operator=( const ImageWriter& );


/// static queries ---------------------------------------------------------------------------------
public:
	static  void getFormatNames( vector<string>& names );
	static  void write( const ImageCieXyz&, const ImageConverter&, dword format, ostream& imageBytes );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getName( string& )                                           const =0;
	virtual void write( const ImageCieXyz&, const ImageConverter&, ostream& ) const =0;


/// fields -----------------------------------------------------------------------------------------
private:
	static PngWriter     pngSrgbWriter_s;
	static PngWriter     pngWriter_s;
	static RealPngWriter realPngWriter_s;

	static const ImageWriter* writers_s[];
	static const dword        noOfWriters_s;

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE[];
};


}}




#endif//ImageWriter_h
