/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef PngWriter_h
#define PngWriter_h


#include "Primitives.h"
#include "ImageWriter.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using std::string;
	using std::ostream;


class PngWriter
 :	public ImageWriter
{
/// standard object services -----------------------------------------------------------------------
public:
	         PngWriter( bool isSRGB );

	virtual ~PngWriter() {}
private:
	         PngWriter( const PngWriter& );
	PngWriter& operator=( const PngWriter& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getName( string& )                                           const;
	virtual void write( const ImageCieXyz&, const ImageConverter&, ostream& ) const;

	static const char* CLASS_NAME();

	static const char EXCEPTION_MESSAGE_1[];
	static const char EXCEPTION_MESSAGE_2[];
	static const char EXCEPTION_MESSAGE_3[];


/// fields -----------------------------------------------------------------------------------------
private:
	bool isSRGB_m;

	static const char FORMAT_NAME_GENERAL[];
	static const char FORMAT_NAME_SRGB[];
	static const char IMAGE_TEXT_SOFTWARE[];
};


}}




#endif//PngWriter_h








/*
www.libpng.org/pub/png
www.colosseumbuilders.com
www.w3.org/Graphics/Color/sRGB

SIGNATURE
IHDR
tEXt
tEXt
tEXt
sRGB
gAMA
cHRM
IDAT
IDAT
IDAT
IEND


n_dword
{
	byte3, byte2, byte1, byte0
}

udword
{
	0 - 7FFFFFFF
}

sdword
{
	80000001 - 7FFFFFFF
}

SIGNATURE
{
	137 80 78 71 13 10 26 10
}

chunk
{
	nudword dataLength
	nudword chunkType
	nbyte[] data
	nudword crcCode
}

IHDR (length= 13)
{
	nudword width
	nudword height
	nubyte  bitDepth          8 (8bit per color channel)
	nubyte  colorType         6 (rgba 32bit) 2 (rgb 24bit)
	nubyte  compressionMethod 0
	nubyte  filterMethod      0
	nubyte  interlaceMethod   0 (1)
}

IEND (length= 0)
{
}

tEXt
{
	nubyte[] keyword {Title Author Description Copyright Software}
	nubyte   null    0
	nubyte[] text (no \0 or controls, newline is \n=0x0A)
}

sRGB (length= 1)
{
	nubyte renderingIntent 0 (perceptual)
}

gAMA (length= 4)
{
   nudword Gamma 45455
}

cHRM (length= 32)
{
   nudword White Point x 31270
   nudword White Point y 32900
   nudword Red x         64000
   nudword Red y         33000
   nudword Green x       30000
   nudword Green y       60000
   nudword Blue x        15000
   nudword Blue y         6000
}

IDAT
{
	scanline
		nubyte filterCode 0
		nubyte r
		nubyte g
		nubyte b
		nubyte a
		...

	filter scanlines
		select the filter that gives the smallest sum of absolute values of outputs
	compress all using deflate
	divide into chunks
}

             Chunk
               |
 -------------------------------
 |    |    |    |    |    |    |
Ihdr Iend Idat Text Srgb Gama Chrm



obytestream
	Crc
	obytestream mustnt write terminating '\0' of strings/char*s
PngChunkIdat
	filters
	deflate
		ftp://ftp.cdrom.com/pub/infozip/
*/
