/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef EyePathNode_h
#define EyePathNode_h


#include "Primitives.h"
#include "SurfacePoint.h"
#include "LightStream.h"
#include <vector>
#include <iosfwd>




#include "perceptuum2_interaction.h"
namespace perceptuum2 { namespace interaction {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Float01;
	using hxa7241::utility::Float01c;
	using std::vector;


class EyePathNode
	: public SurfacePoint
{
/// standard object services -----------------------------------------------------------------------
public:
	         EyePathNode();
	         EyePathNode( const Intersection&       intersection,
	                      const ObjectRenderableId& outsideObject );

	virtual ~EyePathNode();
	         EyePathNode( const EyePathNode& );
	EyePathNode& operator=( const EyePathNode& );

	friend  std::ostream& operator<<( std::ostream&, const EyePathNode& );


/// queries ----------------------------------------------------------------------------------------
	virtual void  getNextPathStep( const Float01       stochastics[7],
	                               const UnitVector3f& outDirection,
	                               Ray&                inRay,
	                               Light&              scaling,
	                               bool&               isDiffuse,
	                               bool&               isDirectSeparate )                     const;

	virtual void  getGatheredLightDistal( const vector<LightStream>& illuminations,
	                                      const UnitVector3f&        outDirection,
	                                      Float01c                   shinynessPoint,
	                                      Light&                     outLight )               const;
	virtual void  getGatheredLightLocal( const vector<LightStream>& illuminations,
	                                     const UnitVector3f&        outDirection,
	                                     Float01c                   shinynessPoint,
	                                     Light&                     outLight )                const;
	virtual void  getEmittedLight( const UnitVector3f& outDirection,
	                               Light&              emitted )                              const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  getGatheredLight( const vector<LightStream>& illuminations,
	                                const UnitVector3f&        outDirection,
	                                Float01c                   shinynessPoint,
	                                bool                       isCos,
	                                Light&                     outLight )                     const;

	virtual void  assign( const SurfacePoint& );
	virtual void  assign( const EyePathNode& );


/// fields -----------------------------------------------------------------------------------------
private:
};




std::ostream& operator<<( std::ostream&, const EyePathNode& );


}}




#endif//EyePathNode_h
