/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Transform.h"
#include <iostream>

#include "Intersection.h"   /// own header is included last


using namespace perceptuum2::modelling;




/// standard object services -----------------------------------------------------------------------
Intersection::Intersection()
{
}


Intersection::~Intersection()
{
}


Intersection::Intersection( const Intersection& other )
 :	position_m( other.position_m ),
	normal_m  ( other.normal_m ),
	coords_m  ( other.coords_m ),
	object_m  ( other.object_m )
{
}


Intersection& Intersection::operator=( const Intersection& other )
{
	if( &other != this )
	{
		position_m = other.position_m;
		normal_m   = other.normal_m;
		coords_m   = other.coords_m;
		object_m   = other.object_m;
	}

	return *this;
}




/// streaming --------------------------------------------------------------------------------------
std::ostream& perceptuum2::modelling::operator<<( std::ostream& out, const Intersection& obj )
{
	return out << "{" << obj.position_m << ", " << obj.normal_m << ", " << obj.coords_m << ", " << obj.object_m << "}";
}




/// commands ---------------------------------------------------------------------------------------
void Intersection::set
(
	const Vector3f&           position,
	const UnitVector3f&       normal,
	const Vector2f&           coords,
	const ObjectRenderableId& object
)
{
	position_m = position;
	normal_m   = normal;
	coords_m   = coords;
	object_m   = object;
}



Intersection& Intersection::transformEq
(
	const Transform& trans
)
{
	trans.transform( position_m );
	trans.transform( normal_m );

	return *this;
}
