/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectDefault_h
#define ObjectDefault_h


#include "Primitives.h"
#include "ObjectRenderable.h"
#include "CieXyz.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;


class ObjectDefault
	: public ObjectRenderable
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectDefault( const Scene* );
//	         ObjectDefault( const Scene*, const CieXyz& color );

	virtual ~ObjectDefault();
	         ObjectDefault( const ObjectDefault& );
	ObjectDefault& operator=( const ObjectDefault& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setMedium( Float01 transparency, float refractiveIndex );
//	virtual void  setColor( const CieXyz& color );

	virtual bool  update( float time );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual bool    getEmitted( const Vector3f& posRands,
	                            vector<float>&  compositionStorage,
	                            Intersection&,
	                            Qualities& )                                                  const;

	virtual bool    getIntersection( const ObjectRenderableId&,
	                                 const Ray&,
	                                 float& distance,
	                                 Intersection& )                                          const;
	virtual void    getObjectsForProjection( const Transclip& transform,
	                                         ClipFlags        clipFlags,
	                                         vector<ProjectableObjectRef>& )                  const;
	virtual void    getMesh( Mesh&, const Mesh** )                                            const;

	virtual void    getQualities( const Intersection&, Qualities& )                           const;

	virtual Float01 getTransparency( const ObjectRenderableId&,
	                                 const Vector3f& position )                               const;
	virtual float   getRefractiveIndex( const ObjectRenderableId&,
	                                    const Vector3f& position )                            const;
	virtual const CieXyz& getEmittance( const ObjectRenderableId& )                           const;

	virtual const char* getClassName()                                                        const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectDefault& );


/// fields -----------------------------------------------------------------------------------------
private:
	CieXyz  color_m;
	Float01 transparency_m;
	float   refractiveIndex_m;
};


}}




#endif//ObjectDefault_h
