/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Serialization.h"
#include "Scene.h"
#include "ObjectMesh.h"
#include "ObjectMeshBlock.h"
#include "ObjectMeshSphere.h"
#include "ObjectMeshCylinder.h"
#include "ObjectInstance.h"
#include "ObjectInstanceMoving.h"
#include "Logger.h"
#include <string>

#include "ObjectInstantiator.h"   /// own header is included last


using namespace perceptuum2::modelling;




/// standard object services -----------------------------------------------------------------------
//ObjectInstantiator::ObjectInstantiator()
// :	objectName_m(),
//	pObject_m( 0 )
//{
//}
//
//
//ObjectInstantiator::~ObjectInstantiator()
//{
//}
//
//
//ObjectInstantiator::ObjectInstantiator( const ObjectInstantiator& )
// :	objectName_m(),
//	pObject_m( 0 )
//{
//}
//
//
//ObjectInstantiator& ObjectInstantiator::operator=( const ObjectInstantiator& )
//{
//	objectName_m.clear();
//	pObject_m = 0;
//
//	return *this;
//}




/// queries ----------------------------------------------------------------------------------------
//ObjectRenderable* ObjectInstantiator::getObject() const
//{
//	return pObject_m;
//}




/// serialization ----------------------------------------------------------------------------------
SerializerOut& perceptuum2::modelling::operator<<( SerializerOut& out, const ObjectInstantiator& obj )
{
	/// write type name
	out.write( "objectType", obj.pObject_m->getClassName() );

	/// write instance name
	out.write( "name", obj.objectName_m );

	/// write object
	out << *(obj.pObject_m);

	return out;
}


SerializerIn& perceptuum2::modelling::input( SerializerIn& in, ObjectInstantiator& obj, Scene& context )
{
	/// read type name
	std::string objectType;
	in.read( "objectType", objectType );


	/// read instance name
	in.read( "name", obj.objectName_m );


	/// do polymorphic create
	{
		ObjectRenderable* pObject = 0;
		if( objectType == ObjectMesh::CLASS_NAME() )
		{
			pObject = new ObjectMesh( &context, in, false );
		}
		else if( objectType == ObjectMeshBlock::CLASS_NAME() )
		{
			pObject = new ObjectMeshBlock( &context, in, false );
		}
		else if( objectType == ObjectMeshSphere::CLASS_NAME() )
		{
			pObject = new ObjectMeshSphere( &context, in, false );
		}
		else if( objectType == ObjectMeshCylinder::CLASS_NAME() )
		{
			pObject = new ObjectMeshCylinder( &context, in, false );
		}
		else if( objectType == ObjectInstance::CLASS_NAME() )
		{
			pObject = new ObjectInstance( &context, in, false );
		}
		else if( objectType == ObjectInstanceMoving::CLASS_NAME() )
		{
			pObject = new ObjectInstanceMoving( &context, in, false );
		}

		hxa7241::utility::Logger() << "readin " << pObject << " " << objectType << " '" << obj.objectName_m << "'  parts " << pObject->getSubPartCount();
		if( pObject == 0 )
		{
			hxa7241::utility::Logger() << hxa7241::utility::Logger::EXC1 << "unknown object type in deserialize stream - ignored";
		}

		obj.pObject_m = pObject;
	}


	/// add name to list last, so subobjects cant refer to parents
	context.addNameOfObject( obj.objectName_m );


	return in;
}


const char* ObjectInstantiator::CLASS_NAME()
{
	static const char CLASS_NAME[] = "perceptuum2::modelling::ObjectInstantiator";
	return CLASS_NAME;
}


const char* ObjectInstantiator::CLASS_DESCRIPTION()
{
	static const char CLASS_DESCRIPTION[] = "an envelope-letter class to polymorphically deserialize an ObjectRenderable instance";
	return CLASS_DESCRIPTION;
}
