/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectMeshBlock_h
#define ObjectMeshBlock_h


#include "Primitives.h"
#include "ObjectMesh.h"
#include "Vector3f.h"
#include "Vector2.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Vector2;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


/**
 * a rectilinear block, made of a triangle mesh.
 */
class ObjectMeshBlock
	: public ObjectMesh
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectMeshBlock( const Scene*, SerializerIn&, bool );
protected:
	         ObjectMeshBlock( const Scene* );
public:
	virtual ~ObjectMeshBlock();
	         ObjectMeshBlock( const ObjectMeshBlock& );
	ObjectMeshBlock& operator=( const ObjectMeshBlock& );


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectMeshBlock& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectMeshBlock& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectMesh& );
	virtual void  assign( const ObjectMeshBlock& );

	static  void  makeBlock( const Vector3f&               dimensions,
	                         bool                          isNet,
	                         const Vector2<word>&          textureCoord,
	                         const vector<Vector2<word> >& textureCoords,
	                         float                         textureScale,
	                         vector<Vector3f>&             vertexs,
	                         vector<TriangleImplicit>&     faces );


/// fields -----------------------------------------------------------------------------------------
private:
	Vector3f      dimensions_m;

	bool          isNet_m;
	Vector2<word> textureCoords_m;
	float         textureScale_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectMeshBlock& );
SerializerIn&  operator>>( SerializerIn&,        ObjectMeshBlock& );


}}




#endif//ObjectMeshBlock_h
