/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectMeshSphere_h
#define ObjectMeshSphere_h


#include "Primitives.h"
#include "ObjectMesh.h"
#include "TriangleImplicit.h"
#include "Vector3f.h"
#include <vector>




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


/**
 * a sphere, made of a triangle mesh.
 */
class ObjectMeshSphere
	: public ObjectMesh
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectMeshSphere( const Scene*, SerializerIn&, bool );
protected:
	         ObjectMeshSphere( const Scene* );
public:
	virtual ~ObjectMeshSphere();
	         ObjectMeshSphere( const ObjectMeshSphere& );
	ObjectMeshSphere& operator=( const ObjectMeshSphere& );


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectMeshSphere& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectMeshSphere& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectMesh& );
	virtual void  assign( const ObjectMeshSphere& );

	static  void  makeSphere( float radius, dword resolution,
	                          vector<Vector3f>& vertexs, vector<TriangleImplicit>& faces );
	static  void  makeTrianglePolyhedron( bool octaOrIcosa, float radius,
	                                      vector<Vector3f>& vertexs, vector<TriangleImplicit>& faces );
	static  void  subdivideTrianglePolyhedron( dword subdivision, float radius,
	                                           vector<Vector3f>& vertexs, vector<TriangleImplicit>& faces );
	static  dword makeEdgeKey( dword, dword );


/// fields -----------------------------------------------------------------------------------------
private:
	float radius_m;
	dword resolution_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectMeshSphere& );
SerializerIn&  operator>>( SerializerIn&,        ObjectMeshSphere& );


}}




#endif//ObjectMeshSphere_h
