/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Transform.h"
#include <iostream>

#include "Ray.h"   /// own header is included last


using namespace perceptuum2::modelling;




/// standard object services -----------------------------------------------------------------------
Ray::Ray()
 :	position_m       (),
	direction_m      (),
	objectLastHit_m  (),
	normalOfLastHit_m(),
	lastDirection_m  (),
	isInside_m       (),
	isTransmission_m ()
{
}


/*Ray::Ray
(
	const Vector3f&     position,
	const UnitVector3f& direction
)
 :	position_m       (),
	direction_m      (),
	objectLastHit_m  (),
	normalOfLastHit_m(),
	lastDirection_m  (),
	isInside_m       (),
	isTransmission_m ()
{
	set( position, direction );
}


Ray::Ray
(
	const Vector3f&           position,
	const UnitVector3f&       direction,
	const UnitVector3f&       normal,
	const ObjectRenderableId& object,
	const bool                isTransmitted
)
 :	position_m       (),
	direction_m      (),
	objectLastHit_m  (),
	normalOfLastHit_m(),
	lastDirection_m  (),
	isInside_m       (),
	isTransmission_m ()
{
	set( position, direction, normal, object, isTransmitted );
}


Ray::Ray
(
	const Vector3f&           position,
	const UnitVector3f&       direction,
	const UnitVector3f&       normal,
	const ObjectRenderableId& object
)
 :	position_m       (),
	direction_m      (),
	objectLastHit_m  (),
	normalOfLastHit_m(),
	lastDirection_m  (),
	isInside_m       (),
	isTransmission_m ()
{
	set( position, direction, normal, object );
}*/


Ray::~Ray()
{
}


Ray::Ray( const Ray& other )
 :	position_m       ( other.position_m ),
	direction_m      ( other.direction_m ),
	objectLastHit_m  ( other.objectLastHit_m ),
	normalOfLastHit_m( other.normalOfLastHit_m ),
	lastDirection_m  ( other.lastDirection_m ),
	isInside_m       ( other.isInside_m ),
	isTransmission_m ( other.isTransmission_m )
{
}


Ray& Ray::operator=( const Ray& other )
{
	if( &other != this )
	{
		position_m        = other.position_m;
		direction_m       = other.direction_m;
		objectLastHit_m   = other.objectLastHit_m;
		normalOfLastHit_m = other.normalOfLastHit_m;
		lastDirection_m   = other.lastDirection_m;
		isInside_m        = other.isInside_m;
		isTransmission_m  = other.isTransmission_m;
	}

	return *this;
}




/// streaming --------------------------------------------------------------------------------------
std::ostream& perceptuum2::modelling::operator<<( std::ostream& out, const Ray& obj )
{
	return out << "{" << obj.position_m << ", " << obj.direction_m << ", " <<
	              obj.objectLastHit_m << ", " << obj.normalOfLastHit_m << ", " <<
	              obj.lastDirection_m << ", " << obj.isInside_m  << ", " << obj.isTransmission_m << "}";
}




/// commands ---------------------------------------------------------------------------------------
void Ray::set
(
	const Vector3f&     position,
	const UnitVector3f& direction
)
{
	set( position, direction, UnitVector3f::ZERO(), ObjectRenderableId(), false );
}


void Ray::set
(
	const Vector3f&           position,
	const UnitVector3f&       direction,
	const UnitVector3f&       normal,
	const ObjectRenderableId& object,
	const bool                isTransmitted
)
{
	position_m        = position;

	lastDirection_m   = direction_m;
	direction_m       = direction;

	normalOfLastHit_m = normal;
	objectLastHit_m   = object;

	isInside_m        = isInside_m ^ isTransmitted;
	isTransmission_m  = isTransmitted;
}


void Ray::set
(
	const Vector3f&           position,
	const UnitVector3f&       direction,
	const UnitVector3f&       normal,
	const ObjectRenderableId& object
)
{
	position_m        = position;

	lastDirection_m   = direction_m;
	direction_m       = direction;

	normalOfLastHit_m = normal;
	objectLastHit_m   = object;


	/// recalc auxilliaries
	isInside_m        = isDirDotNormNeg();
	isTransmission_m  = calcIsTransmission();
}


/*void Ray::setPosition( const Vector3f& position )
{
	position_m = position;
}


void Ray::setDirection( const UnitVector3f& direction )
{
	lastDirection_m = direction_m;
	direction_m     = direction;

	/// recalc auxilliaries
	isInside_m        = isDirDotNormNeg();
	isTransmission_m  = calcIsTransmission();
}


void Ray::setNormal( const UnitVector3f& normal )
{
	normalOfLastHit_m = normal;

	/// recalc auxilliaries
	isInside_m        = isDirDotNormNeg();
	isTransmission_m  = calcIsTransmission();
}


void Ray::setObject( const ObjectRenderableId& object )
{
	objectLastHit_m = object;
}*/


Ray& Ray::transformEq
(
	const Transform& trans
)
{
	trans.transform( position_m );
	trans.transform( direction_m );
	trans.transform( normalOfLastHit_m );
	trans.transform( lastDirection_m );

	return *this;
}




/// queries ----------------------------------------------------------------------------------------
bool Ray::isTransmission() const
{
	return isTransmission_m;
}


bool Ray::isInside() const
{
	return isInside_m;
}


void Ray::getPosition
(
	const float distance,
	Vector3f&   position
) const
{
	position  = direction_m.getVector();
	position *= distance;
	position += position_m;
}


Vector3f Ray::getPosition
(
	const float distance
) const
{
	return Vector3f( (direction_m.getVector() * distance) += position_m );
}




/// implementation ---------------------------------------------------------------------------------
bool Ray::isDirDotNormNeg() const
{
	return direction_m.dot( normalOfLastHit_m ) < 0.0f;
}


bool Ray::calcIsTransmission() const
{
	/// if sign(normal.prevDir) == sign(normal.dir), transmission
	/// if transmission and sign is neg, entry
	/// if not transmission and sign is neg, inside

	return (!lastDirection_m.isZero()) &&
	       (isInside_m == (lastDirection_m.dot( normalOfLastHit_m ) < 0.0f));
}
