/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef TriangleImplicit_h
#define TriangleImplicit_h


#include "Primitives.h"
#include "Vector3f.h"
#include "Vector2.h"
#include <vector>

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Vector2;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


class TriangleImplicit
{
/// standard object services -----------------------------------------------------------------------
public:
	         TriangleImplicit();
	         TriangleImplicit( word vertexIndex0,
	                           word vertexIndex1,
	                           word vertexIndex2,
	                           bool interpolateNormal );

	        ~TriangleImplicit();
	         TriangleImplicit( const TriangleImplicit& );
	TriangleImplicit& operator=( const TriangleImplicit& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( word vertexIndex0,
	                   word vertexIndex1,
	                   word vertexIndex2 );
	        void  set( bool interpolateNormal );
	        void  set( const Vector2<word> txy[3] );

			void  reverseOrientation();


/// queries ----------------------------------------------------------------------------------------
public:
			bool  checkIndexesRange( dword vertexCount )                                      const;
			bool  checkIndexesDistinct()                                                      const;
	        void  getNormal( const vector<Vector3f>& vertexPositions,
	                         UnitVector3f&           normal )                                 const;
	        float getArea( const vector<Vector3f>& vertexPositions )                          const;

	        dword getVertexIndex0()                                                           const;
	        dword getVertexIndex1()                                                           const;
	        dword getVertexIndex2()                                                           const;

	        bool  getInterpolateNormal()                                                      const;

	        void  getTextureCoords( Vector2<word> txy[3] )                                    const;

	        dword getCommonVertexs( const TriangleImplicit& other,
	                                bool& isFirst,
	                                bool& isSecond,
	                                bool& isThird )                                           const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const TriangleImplicit& );
	friend  SerializerIn&  operator>>( SerializerIn&,        TriangleImplicit& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	        void  zeroTextureCoords();


/// fields -----------------------------------------------------------------------------------------
private:
	/// 20 bytes (need to review for each compiler)
	int index0_m : 16;
	int index1_m : 16;
	int index2_m : 16;

	unsigned int interpolateNormal_m : 1;

	word txys_m[3][2];


	/*/// 16 bytes
	/// (16 16) (16 16) (16 6 10) (10 10 10 2)
	int index0_m : 16;
	int index1_m : 16;
	int index2_m : 16;

	int txOffset_m : 16;
	int tyOffset_m : 16;

	unsigned int tx1Sign_m : 1;
	unsigned int ty1Sign_m : 1;
	unsigned int tx2Sign_m : 1;
	unsigned int ty2Sign_m : 1;

	int tx1_m : 10;
	int ty1_m : 10;
	int tx2_m : 10;
	int ty2_m : 10;

	unsigned int interpolateNormal_m : 1;*/
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const TriangleImplicit& );
SerializerIn&  operator>>( SerializerIn&,        TriangleImplicit& );




/// inlines ///
inline
dword TriangleImplicit::getVertexIndex0() const
{
	return dword( index0_m );
}


inline
dword TriangleImplicit::getVertexIndex1() const
{
	return dword( index1_m );
}


inline
dword TriangleImplicit::getVertexIndex2() const
{
	return dword( index2_m );
}


inline
bool TriangleImplicit::getInterpolateNormal() const
{
	return interpolateNormal_m;
}


}}




#endif//TriangleImplicit_h
