/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef EmitterPath_h
#define EmitterPath_h


#include "Primitives.h"
#include "EmitterPathNode.h"
#include "Float01c.h"
#include <vector>

#include "perceptuum2_interaction.h"
#include "perceptuum2_modelling.h"
#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::interaction::Light;
	using perceptuum2::interaction::LightStream;
	using perceptuum2::interaction::EmitterPathNode;
	using perceptuum2::interaction::EyePathNode;
	using perceptuum2::modelling::ObjectRenderableId;
	using perceptuum2::modelling::Intersection;
	using hxa7241::utility::Float01;
	using hxa7241::utility::Float01c;
	using std::vector;


class EmitterPath
{
/// standard object services -----------------------------------------------------------------------
public:
	         EmitterPath();
	         EmitterPath( dword    pathLengthLimit,
	                      Float01c pathEndThreshold,
	                      bool     isRoulette );

	virtual ~EmitterPath();
	         EmitterPath( const EmitterPath& );
	EmitterPath& operator=( const EmitterPath& );


/// commands ---------------------------------------------------------------------------------------
	virtual void  set( dword    pathLengthLimit,
	                   Float01c pathEndThreshold,
	                   bool     isRoulette );
	virtual void  trace( const SpacetimeIndex&         sceneIndex,
	                     const SamplingControlEmitter& emitterSamplingControl,
	                     vector<float>&                lightsourceCompositionStorage,
	                     float                         emittanceScaling );


/// queries ----------------------------------------------------------------------------------------
	virtual void  getSpreadLight( const SpacetimeIndex& sceneIndex,
	                              const EyePathNode&    lastEyePathNode,
	                              Float01c              shinynessPoint,
	                              vector<LightStream>&  spreadStorage )                       const;
	virtual dword getLength()                                                                 const;
	virtual const vector<EmitterPathNode>& getNodes()                                         const;
	virtual const vector<EmitterPathNode>& getNodes( dword& length )                          const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual bool  chooseLightSourceStart( const ObjectRenderableId&     rootObject,
	                                      const SamplingControlEmitter& samplingControl,
	                                      vector<float>&                lightsourceCompositionStorage,
	                                      Intersection&                 intersection,
	                                      Light&                        emittance,
	                                      LightStream&                  firstPathStep,
	                                      bool&                         isTransmitted,
	                                      bool&                         isDiffuse )           const;

	virtual void  assign( const EmitterPath& );


/// fields -----------------------------------------------------------------------------------------
private:
	dword    pathLengthLimit_m;
	Float01c pathEndThreshold_m;
	bool     isRoulette_m;

	vector<EmitterPathNode> nodes_m;
	dword                   nodesLength_m;
};


}}




#endif//EmitterPath_h
