/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef EyePath_h
#define EyePath_h


#include "Primitives.h"
#include "Light.h"
#include "Float01c.h"

#include "perceptuum2_interaction.h"
#include "perceptuum2_modelling.h"
#include "hxa7241_graphics.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::interaction::EyePathNode;
	using perceptuum2::interaction::Light;
	using perceptuum2::modelling::Ray;
	using hxa7241::utility::Float01;
	using hxa7241::utility::Float01c;


class EyePath
{
/// standard object services -----------------------------------------------------------------------
public:
	         EyePath( dword    pathLengthLimit,
	                  Float01c pathEndThreshold );

	virtual ~EyePath();
private:
	         EyePath( const EyePath& );
	EyePath& operator=( const EyePath& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  trace( const SamplingControlPath&,
	                     const SamplingControlDiffuse&,
	                     const SpacetimeIndex&,
	                     const Illumination&,
	                     const Ray& );
	virtual void  traceFromFirstHit( const SamplingControlPath&,
	                                 const SamplingControlDiffuse&,
	                                 const SpacetimeIndex&,
	                                 const Illumination&,
	                                 const EyePathNode& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void  getEyeEndLight( bool& isHit, Light& light )                                 const;
	virtual void  getEyeEndLight( Light& light )                                              const;
	virtual dword getPathLength()                                                             const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  makeIlluminationFromPath( const SamplingControlPath&    pathSamplingControl,
	                                        const SamplingControlDiffuse& diffuseSamplingControl,
	                                        const SpacetimeIndex&         sceneIndex,
	                                        const Illumination&           illumination,
	                                        const Ray&                    initialRay );


/// fields -----------------------------------------------------------------------------------------
private:
	const dword    pathLengthLimit_m;
	const Float01c pathEndThreshold_m;

//	vector<EyePathNode> nodes_m;
	dword               pathLength_m;

	Light eyeEndLight_m;
};


}}




#endif//EyePath_h
