/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Ray.h"
#include <iostream>

#include "MediaStack.h"   /// own header is included last


using namespace perceptuum2::transport;




/// standard object services -----------------------------------------------------------------------
MediaStack::MediaStack
(
	const ObjectRenderableId& defaultObject
)
 :	stack_m()
{
	stack_m.reserve( 8 );
	push( defaultObject );
}


MediaStack::~MediaStack()
{
}


MediaStack::MediaStack( const MediaStack& other )
{
	MediaStack::operator=( other );
}


MediaStack& MediaStack::operator=( const MediaStack& other )
{
	if( &other != this )
	{
		stack_m = other.stack_m;
	}

	return *this;
}




/// streaming --------------------------------------------------------------------------------------
std::ostream& perceptuum2::transport::operator<<( std::ostream& out, const MediaStack& obj )
{
	out << "{";
	for( dword i = obj.stack_m.size();  i-- > 0; )
	{
		out << obj.stack_m[i] << (i != 0 ? "," : "");
	}
	out << "}";

	return out;
}




/// commands ---------------------------------------------------------------------------------------
void MediaStack::push
(
	const ObjectRenderableId& object
)
{
	stack_m.push_back( object );
}


void MediaStack::pop()
{
	/// always leave at least one item on the stack
	if( stack_m.size() > 1 )
	{
		stack_m.pop_back();
	}
}


void MediaStack::update
(
	const Ray& ray
)
{
	if( ray.isTransmission() )
	{
		if( ray.isInside() )
		{
			push( ray.getObject() );
		}
		else
		{
			pop();
		}
	}
}


//void MediaStack::update
//(
//	const bool              isTransmit,
//	const bool              isEntry,
//	const ObjectRenderable& object
//)
//{
//	if( isTransmit )
//	{
//		if( isEntry )
//		{
//			push( object );
//		}
//		else
//		{
//			pop();
//		}
//	}
//}




/// queries ----------------------------------------------------------------------------------------
const ObjectRenderableId& MediaStack::getTop() const
{
	return stack_m.back();
}


ObjectRenderableId MediaStack::getOutsideObject
(
	const Vector3f&           inDirection,
	const Vector3f&           surfaceNormal,
	const ObjectRenderableId& insideObject
) const
{
	/// set for common cases: ray reflection or entry
	ObjectRenderableId outsideObject = stack_m.back();

	/// check if ray is from inside
	const bool isFromInside = inDirection.dot( surfaceNormal ) >= 0.0f;
	if( isFromInside )
	{
		/// get object below top of stack
		const int topMinusOne = stack_m.size() - 2;
		const ObjectRenderableId lastObject = stack_m[ topMinusOne >= 0 ? topMinusOne : 0 ];

		/// use the last object, unless the [ (  ->] ) overlapping case obtains
		if( !lastObject.isMainObjectsEqual( insideObject ) )
		{
			outsideObject = lastObject;
		}
	}

	return outsideObject;
}
