/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef MediaStack_h
#define MediaStack_h


#include "Primitives.h"
#include "ObjectRenderableId.h"
#include <vector>
#include <iosfwd>

#include "perceptuum2_modelling.h"
#include "hxa7241_graphics.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Ray;
	using perceptuum2::modelling::ObjectRenderableId;
	using hxa7241::graphics::Vector3f;
	using std::vector;


class MediaStack
{
/// standard object services -----------------------------------------------------------------------
public:
	         MediaStack( const ObjectRenderableId& );

	        ~MediaStack();
	         MediaStack( const MediaStack& );
	MediaStack& operator=( const MediaStack& );

	friend  std::ostream& operator<<( std::ostream&, const MediaStack& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  push( const ObjectRenderableId& );
	        void  pop();
	        void  update( const Ray& );
	        //void  update( bool isTransmit, bool isEntry, const ObjectRenderableId& object );


/// queries ----------------------------------------------------------------------------------------
public:
	        const ObjectRenderableId& getTop()                                                   const;
	        ObjectRenderableId        getOutsideObject( const Vector3f&           inDirection,
	                                                    const Vector3f&           surfaceNormal,
	                                                    const ObjectRenderableId& insideObject ) const;


/// fields -----------------------------------------------------------------------------------------
private:
	vector<ObjectRenderableId> stack_m;
};




std::ostream& operator<<( std::ostream&, const MediaStack& );


}}




#endif//MediaStack_h
