/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Photon_h
#define Photon_h


#include "Primitives.h"
#include "XyzE.h"
#include "Vector3f.h"

#include "hxa7241_graphics.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::XyzE;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;


/**
 * a photon interaction with a scattering event.<br/><br/>
 *
 * groups together some data, compressed a little.<br/><br/>
 *
 * data comprises: light energy, position, incoming direction, and id of last
 * object.
 */
class Photon
{
/// standard object services -----------------------------------------------------------------------
public:
	         Photon();
	         Photon( const CieXyz&       power,
	                 const Vector3f&     position,
	                 const UnitVector3f& direction,
	                 dword               id );

	        ~Photon();
	         Photon( const Photon& );
	Photon& operator=( const Photon& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( const CieXyz&       power,
	                   const Vector3f&     position,
	                   const UnitVector3f& direction,
	                   dword               id );
	        void  scalePower( float );


/// queries ----------------------------------------------------------------------------------------
public:
	        void  get( CieXyz&       power,
	                   Vector3f&     position,
	                   UnitVector3f& direction,
	                   dword&        id )                                                     const;

	        const Vector3f& getPosition()                                                     const;


/// implementation ---------------------------------------------------------------------------------
private:
	static  void  initAngleTables( float cosTheta[256],
	                               float sinTheta[256],
	                               float cosPhi[256],
	                               float sinPhi[256] );

	static  void  compressDirection( const UnitVector3f&,
	                                 byte& theta,
	                                 byte& phi );
	static  void  decompressDirection( byte theta,
	                                   byte phi,
	                                   UnitVector3f& );


/// fields -----------------------------------------------------------------------------------------
private:
	/// 24 bytes (need to review for each compiler)
	XyzE         power_m;
	Vector3f     position_m;
	unsigned int theta_m      :  8;
	unsigned int phi_m        :  8;
	unsigned int previousId_m : 32;


	/// constants
	static float COS_THETA_STORE[256];
	static float SIN_THETA_STORE[256];
	static float COS_PHI_STORE[256];
	static float SIN_PHI_STORE[256];

	static const float*const COS_THETA;
	static const float*const SIN_THETA;
	static const float*const COS_PHI;
	static const float*const SIN_PHI;
};








/// inlines ///

/// standard object services -----------------------------------------------------------------------
inline
Photon::Photon
(
	const CieXyz&       power,
	const Vector3f&     position,
	const UnitVector3f& direction,
	const dword         id
)
{
	Photon::set( power, position, direction, id );
}


inline
Photon::~Photon()
{
}


inline
Photon::Photon( const Photon& other )
{
	Photon::operator=( other );
}


}}




#endif//Photon_h
