/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef SamplingControl_h
#define SamplingControl_h


#include "Primitives.h"
#include "HaltonSequence.h"
#include "SamplingControlEmitter.h"
#include "SamplingControlPath.h"
#include "SamplingControlDiffuse.h"
#include "Float01.h"
#include "Vector2.h"

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector2;
	using hxa7241::utility::Float01;
	using hxa7241::utility::HaltonSequence;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


class SamplingControl
{
/// standard object services -----------------------------------------------------------------------
public:
	         SamplingControl();

	virtual ~SamplingControl();
	         SamplingControl( const SamplingControl& );
	SamplingControl& operator=( const SamplingControl& );


/// commands ---------------------------------------------------------------------------------------
	virtual void  advanceFrame();
	virtual void  advanceIllumination0();
	virtual void  advanceIllumination1();


/// queries ----------------------------------------------------------------------------------------
	virtual Float01          getTimePoint()                                                   const;
	virtual Vector2<Float01> getPixelPoint()                                                  const;
	virtual Vector2<Float01> getLensPoint()                                                   const;

	virtual const SamplingControlEmitter& getEmitterControl0()                                const;
	virtual const SamplingControlEmitter& getEmitterControl1()                                const;
	virtual const SamplingControlPath&    getEyePathControl()                                 const;
	virtual const SamplingControlDiffuse& getDiffuseControl()                                 const;

	virtual dword getFrameIndex()                                                             const;


	/// test
	static  bool  test( bool isLogging );


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const SamplingControl& );
	friend  SerializerIn&  operator>>( SerializerIn&,        SamplingControl& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const SamplingControl& );


/// fields -----------------------------------------------------------------------------------------
private:
	enum { TIME, PIXEL_X, PIXEL_Y, LENS_X, LENS_Y };

	HaltonSequence         haltons_m[6];
	SamplingControlEmitter emitterControl0_m;
	SamplingControlEmitter emitterControl1_m;
	SamplingControlPath    eyePathControl_m;
	SamplingControlDiffuse diffuseControl_m;


	/// constants
	static const dword HALTON_BASES[];
	static const dword SOBOL_DIMENSIONS_EMIT_00[];
	static const dword SOBOL_DIMENSIONS_EMIT_01[];
	static const dword RANDOM_SEEDS_EMIT_0[];
	static const dword SPECIAL_LEVELS_EMIT_0;
	static const dword SOBOL_DIMENSIONS_EMIT_10[];
	static const dword SOBOL_DIMENSIONS_EMIT_11[];
	static const dword RANDOM_SEEDS_EMIT_1[];
	static const dword SPECIAL_LEVELS_EMIT_1;
	static const dword SOBOL_DIMENSIONS_EYE[];
	static const dword RANDOM_SEEDS_EYE[];
	static const dword SPECIAL_LEVELS_EYE;
	static const dword SOBOL_DIMENSIONS_DIFFUSE[];
	static const dword RANDOM_SEEDS_DIFFUSE[];
	static const dword SPECIAL_LEVELS_DIFFUSE;
	static const dword SEEDS[];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const SamplingControl& );
SerializerIn&  operator>>( SerializerIn&,        SamplingControl& );


}}




#endif//SamplingControl_h
