/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef SamplingControlEmitter_h
#define SamplingControlEmitter_h


#include "Primitives.h"
#include "SamplingControlPath.h"
#include "SobolSequence.h"
#include "Float01.h"
#include "Vector2.h"

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Vector2;
	using hxa7241::utility::Float01;
	using hxa7241::utility::SobolSequence;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	typedef Vector2<Float01> Vector2f01;


/**
 * emitter sampling controller.<br/><br/>
 *
 * provides sampling coordinates for initial emission position and direction,
 * and all interactions along a path.<br/><br/>
 */
class SamplingControlEmitter
{
public:
	static const dword SOBOL_COUNT = 5;

/// standard object services -----------------------------------------------------------------------
public:
	         /** @param sobolDimensions1 must have length == SamplingControlPath::SOBOL_COUNT * specialLevels
	             @param randomSeeds must have length == SamplingControlPath::RANDOM_COUNT * specialLevels */
	         SamplingControlEmitter( const dword sobolDimensions0[SOBOL_COUNT],
	                                 const dword sobolDimensions1[],
	                                 const dword randomSeeds[],
	                                 dword       specialLevels,
	                                 dword       extraSeed );

	        ~SamplingControlEmitter();
	         SamplingControlEmitter( const SamplingControlEmitter& );
	SamplingControlEmitter& operator=( const SamplingControlEmitter& );


/// commands ---------------------------------------------------------------------------------------
	        void  advance();


/// queries ----------------------------------------------------------------------------------------
	        Vector3f   getPositionCoords()                                                    const;
	        Vector2f01 getDirectionCoords()                                                   const;
	        void       getInteractionCoords( dword   pathNodeIndex,
	                                      Float01 coords[SamplingControlPath::COORDS_COUNT] ) const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const SamplingControlEmitter& );
	friend  SerializerIn&  operator>>( SerializerIn&,        SamplingControlEmitter& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	enum { POS_X, POS_Y, POS_R, DIR_X, DIR_Y };

	SobolSequence       sobols_m[SOBOL_COUNT];
	SamplingControlPath pathControl_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const SamplingControlEmitter& );
SerializerIn&  operator>>( SerializerIn&,        SamplingControlEmitter& );


}}




#endif//SamplingControlEmitter_h
