/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Vertex.h"
#include "Projector.h"

#include "RasterizationVertex.h"   /// own header is included last


using namespace perceptuum2::transport::projection;




/// standard object services -----------------------------------------------------------------------
RasterizationVertex::RasterizationVertex()
 :	worldPosition_m(),
	screenPosition_m(),
	clipFlags_m()
{
}


RasterizationVertex::RasterizationVertex
(
	const Vertex&    vertex,
	const Vector3f&  meshPosition,
	const Vector3f&  meshScale,
	const ClipFlags  objectClipflags,
	const Projector& projector
)
 :	worldPosition_m(),
	screenPosition_m(),
	clipFlags_m()
{
	set( vertex, meshPosition, meshScale, objectClipflags, projector );
}


RasterizationVertex::~RasterizationVertex()
{
}


RasterizationVertex::RasterizationVertex
(
	const RasterizationVertex& other
)
 :	worldPosition_m ( other.worldPosition_m  ),
	screenPosition_m( other.screenPosition_m ),
	clipFlags_m     ( other.clipFlags_m      )
{
}


RasterizationVertex& RasterizationVertex::operator=
(
	const RasterizationVertex& other
)
{
	if( &other != this )
	{
		worldPosition_m  = other.worldPosition_m;
		screenPosition_m = other.screenPosition_m;
		clipFlags_m      = other.clipFlags_m;
	}

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
/*void RasterizationVertex::set
(
	const Vertex&    vertex,
	const Vector3f&  meshPosition,
	const Vector3f&  meshScale,
	const Projector& projector
)
{
	/// extract model position
	Vector3f modelPosition;
	vertex.getPosition( meshPosition, meshScale, modelPosition );

	/// transform to reqired spaces
	projector.transformModelToWorld( modelPosition, worldPosition_m );
	projector.transformModelToScreen( modelPosition, screenPosition_m );

	clipFlags_m.zero();
}*/


void RasterizationVertex::set
(
	const Vertex&    vertex,
	const Vector3f&  meshPosition,
	const Vector3f&  meshScale,
	const ClipFlags  objectClipflags,
	const Projector& projector
)
{
	/// extract model position
	Vector3f modelPosition;
	vertex.getPosition( meshPosition, meshScale, modelPosition );

	/// transform to reqired spaces
	projector.transformModelToWorld( modelPosition, worldPosition_m );
	projector.transformModelToScreen( objectClipflags, modelPosition, screenPosition_m, clipFlags_m );
}
