/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "perceptuum2.rc.h"
#include "Perceptuum2.h"

#include "StopDialog.h"   /// own header is included last


using namespace perceptuum2::platform_ui;




/// standard object services -----------------------------------------------------------------------
StopDialog::StopDialog
(
	HINSTANCE    hInstance,
	HWND         hWndParent,
	Perceptuum2& p2
)
 :	Win32Dialog( hInstance, hWndParent ),
	p2_m( &p2 ),
	timerId_m( 0 ),
	stop_m( STOP_CANCEL )
{
}


StopDialog::~StopDialog()
{
}




/// commands ---------------------------------------------------------------------------------------
int StopDialog::run()
{
	return Win32Dialog::run();
}




/// overrides --------------------------------------------------------------------------------------
int StopDialog::getDialogResourceId() const
{
	return IDD_STOP;
}


bool StopDialog::dialogProc
(
	HWND   hWndDlg,
	UINT   message,
	WPARAM wParam,
	LPARAM //lParam
)
{
	bool didHandleMessage = true;

	switch( message )
	{
	case WM_INITDIALOG :
		{
			/// request stop at frame end
			p2_m->stopWhenFrameDone();
			stop_m = STOP_FRAME;
			setText( hWndDlg, IDS_STOP_END_FRAME );

			/// start timer
			timerId_m = ::SetTimer( hWndDlg, TIMER_ID, TIMER_PERIOD_MILLISECS, 0 );
			if( timerId_m == 0 )
			{
				p2_m->stop();
				stop_m = STOP_MIDDLE;
				::EndDialog( hWndDlg, stop_m );
			}

			break;
		}
	case WM_COMMAND :
		{
			switch( wParam )
			{
			case IDB_STOP :
				{
					p2_m->stop();
					stop_m = STOP_MIDDLE;

					setText( hWndDlg, IDS_STOP_MID_FRAME );

					/// disable stop button
					HWND hWndButton = ::GetDlgItem( hWndDlg, IDB_STOP );
					::EnableWindow( hWndButton, FALSE );

					/// set focus to cancel button
					HWND hWndCancel = ::GetDlgItem( hWndDlg, IDCANCEL );
					::SetFocus( hWndCancel );

					break;
				}
			case IDB_STOP_FORCIBLY :
				{
					p2_m->stopForcibly();
					stop_m = STOP_FORCE;
					endDialog( hWndDlg );

					break;
				}
			case IDCANCEL :
				{
					p2_m->unStop();
					stop_m = STOP_CANCEL;
					endDialog( hWndDlg );

					break;
				}
			default :
				{
					didHandleMessage = false;
					break;
				}
			}
			break;
		}
	case WM_TIMER :
		{
			if( !p2_m->isThreadRunning() )
			{
				endDialog( hWndDlg );
			}
			break;
		}
	default:
		didHandleMessage = false;
	}

	return didHandleMessage;
}




/// implementation ---------------------------------------------------------------------------------
void StopDialog::setText
(
	HWND hWndDlg,
	const int id
)
{
	TCHAR buf[128];
	const int c = ::LoadString( hInstance_m, id, buf, sizeof(buf)/sizeof(TCHAR) );

	if( c > 0 )
	{
		::SetDlgItemText( hWndDlg, IDC_STOP_STATUS, buf );
	}
}


void StopDialog::endDialog
(
	HWND hWndDlg
)
{
	::KillTimer( hWndDlg, timerId_m );
	timerId_m = 0;

	::EndDialog( hWndDlg, stop_m );
}
