/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ByteSerializerIn_h
#define ByteSerializerIn_h


#include "Primitives.h"
#include "SerializerIn.h"
#include <iosfwd>
#include <fstream>
#include <string>




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::istream;
	using std::string;


/**
 * a binary file inward serializer.<br/><br/>
 *
 * the object in the byte file must match the receiving object type exactly.
 * <br/><br/>
 *
 * the outward serializer of the byte file must match this inward serializer
 * version.
 *
 * @see ByteSerializerOut
 */
class ByteSerializerIn
	: public SerializerIn
{
/// standard object services -----------------------------------------------------------------------
private:
	         ByteSerializerIn( istream&, const char* fileName );

	        ~ByteSerializerIn();
	         ByteSerializerIn( const ByteSerializerIn& );
	ByteSerializerIn& operator=( const ByteSerializerIn& );


/// commands ---------------------------------------------------------------------------------------
public:
	template<class TYPE>
	static  bool  serializeIn( TYPE& object, const char* filePathname );
	template<class TYPE>
	static  bool  serializeIn( TYPE& object, istream&, const char* fileName );


/// queries ----------------------------------------------------------------------------------------
public:


/// implementation ---------------------------------------------------------------------------------
private:
	/// overrides
	virtual ByteSerializerIn& operator>>( void*& );
	virtual ByteSerializerIn& operator>>( bool& );
	virtual ByteSerializerIn& operator>>( hxa7241::utility::byte& );
	virtual ByteSerializerIn& operator>>( ubyte& );
	virtual ByteSerializerIn& operator>>( word& );
	virtual ByteSerializerIn& operator>>( uword& );
	virtual ByteSerializerIn& operator>>( dword& );
	virtual ByteSerializerIn& operator>>( udword& );
	virtual ByteSerializerIn& operator>>( long& );
	virtual ByteSerializerIn& operator>>( unsigned long& );
	virtual ByteSerializerIn& operator>>( float& );
	virtual ByteSerializerIn& operator>>( double& );
	virtual ByteSerializerIn& operator>>( char& );
	virtual ByteSerializerIn& operator>>( char* );
	virtual ByteSerializerIn& operator>>( string& );

	virtual void  readHeader( const char* name );
//	virtual void  readFooter( const char* name );

//	virtual bool  readPreObject ( const char* name );
//	virtual void  readPostObject( const char* name );

	static  void  checkHeaderLine( istream&, const char* pLine, const char* pMessage );


/// fields -----------------------------------------------------------------------------------------
private:

	static const char EXCEPTION_MESSAGE1[];
	static const char EXCEPTION_MESSAGE2[];

	static const char CLASS_NAME[];
};








/// TEMPLATES ///


/// commands ---------------------------------------------------------------------------------------
/// static
template<class TYPE>
bool ByteSerializerIn::serializeIn
(
	TYPE&            object,
	const char*const filePathname
)
{
	std::ifstream file( filePathname, std::ifstream::binary );

	return ByteSerializerIn::serializeIn( object, file, filePathname );
}


template<class TYPE>
bool ByteSerializerIn::serializeIn
(
	TYPE&            object,
	istream&         in,
	const char*const fileName
)
{
	ByteSerializerIn bsi( in, fileName );

	return bsi.read( object );
}


}}




#endif//ByteSerializerIn_h
