/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Exception.h"


/// for borland c++ 5.5.1 (another at the end of the file)
#ifdef BORLANDCPP551
#pragma option push -b -a8 -pc -Vx- -Ve- -w-inl -w-aus -w-sig
#endif


using namespace hxa7241::utility;




/// statics
const char Exception::DEFAULT_MESSAGE[] = "unknown exception";
const char Exception::DEFAULT_CLASS[]   = "unknown-class";
const char Exception::DEFAULT_METHOD[]  = "unknown-method";




/// standard object services -----------------------------------------------------------------------
Exception::Exception
(
	const char* pMessage,
	const char* pClassName,
	const char* pMethodName
) throw()
 :	exception(),
	pMessage_m(),
	pClassName_m( pClassName ),
	pMethodName_m( pMethodName )
{
	if( pMessage == 0 )
	{
		pMessage = DEFAULT_MESSAGE;
	}
	copyString( pMessage, pMessage_m, sizeof(pMessage_m)/sizeof(*pMessage_m) );

	if( pClassName_m == 0 )
	{
		pClassName_m = DEFAULT_CLASS;
	}
	if( pMethodName_m == 0 )
	{
		pMethodName_m = DEFAULT_METHOD;
	}
}


Exception::Exception() throw()
 :	exception(),
	pMessage_m(),
	pClassName_m( DEFAULT_CLASS ),
	pMethodName_m( DEFAULT_METHOD )
{
	copyString( DEFAULT_MESSAGE, pMessage_m, sizeof(pMessage_m)/sizeof(*pMessage_m) );
}


Exception::~Exception() throw()
{
}


Exception::Exception( const Exception& other ) throw()
 :	exception( other ),
	pMessage_m(),
	pClassName_m( other.pClassName_m ),
	pMethodName_m( other.pMethodName_m )
{
	copyString( other.pMessage_m, pMessage_m, sizeof(pMessage_m)/sizeof(*pMessage_m) );
}


Exception& Exception::operator=( const Exception& other ) throw()
{
	assign( other );

	return *this;
}




/// queries ----------------------------------------------------------------------------------------
const char* Exception::what() const throw()
{
	const char* pWhat = "";

	try
	{
		makeWhat( store_m );
		pWhat = store_m.c_str();
	}
	catch( ... )
	{
		pWhat = pMessage_m;
	}

	return pWhat;
}




/// implementation ---------------------------------------------------------------------------------
string& Exception::makeWhat( string& s ) const
{
	s  = "";
	appendMessage( s );
	s += "\n\t";
	appendOrigin( s );

	return s;
}


string& Exception::appendMessage( string& s ) const
{
	s += pMessage_m;

	return s;
}


string& Exception::appendOrigin( string& s ) const
{
	s += "[ ";
	s += pClassName_m;
	s += ".";
	s += pMethodName_m;
	s += " ]";

	return s;
}


void Exception::copyString
(
	const char* pSource,
	char*       pTarget,
	const dword targetLength )
{
	for( dword i = 1;  (*pSource != 0) & (i < targetLength);  ++i )
	{
		*(pTarget++) = *(pSource++);
	}
	*pTarget = 0;
}


void Exception::assign
(
	const Exception& other
)
{
	if( &other != this )
	{
		exception::operator=( other );

		copyString( other.pMessage_m, pMessage_m, sizeof(pMessage_m)/sizeof(*pMessage_m) );
		pClassName_m  = other.pClassName_m;
		pMethodName_m = other.pMethodName_m;
	}
}




#ifdef BORLANDCPP551
#pragma option pop
#endif
