/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Exception_h
#define Exception_h


#include "Primitives.h"
#include <string>
#include <exception>


/// for borland c++ 5.5.1 (another at the end of the file)
#ifdef BORLANDCPP551
#pragma option push -b -a8 -pc -Vx- -Ve- -w-inl -w-aus -w-sig
#endif




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::exception;
	using std::string;


/**
 * a std::exception enhancement, adding extra message fields.
 */
class Exception
	: public exception
{
/// standard object services -----------------------------------------------------------------------
public:
	           /**
	            * the class and method args must be pointers to statics.
	            */
	           Exception( const char* pMessage, const char* pClassName, const char* pMethodName ) throw();
	           Exception()                                                                        throw();

	virtual   ~Exception()                   throw();
	           Exception( const Exception& ) throw();
	Exception& operator=( const Exception& ) throw();


/// queries ----------------------------------------------------------------------------------------
public:
	virtual const char* what() const throw();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual string&     makeWhat( string& )      const;
	virtual string&     appendMessage( string& ) const;
	virtual string&     appendOrigin( string& )  const;

	static  void        copyString( const char* source, char* target, dword targetLength );

	virtual void        assign( const Exception& );


/// fields -----------------------------------------------------------------------------------------
private:
	      char  pMessage_m[256];
	const char* pClassName_m;
	const char* pMethodName_m;

	mutable string store_m;

protected:
	/// constants
	static const char DEFAULT_MESSAGE[];
	static const char DEFAULT_CLASS[];
	static const char DEFAULT_METHOD[];
};


}}




#ifdef BORLANDCPP551
#pragma option pop
#endif


#endif//Exception_h
