/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Float01_h
#define Float01_h


#include "Primitives.h"




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a fp value between 0 and almost 1, ie: [0,1).<br/><br/>
 *
 * 'almost one' equals FLOAT_ALMOST_ONE, which should equal (1.0f - FLT_EPSILON).
 * <br/><br/>
 *
 * provides implicit conversion to and from float.
 *
 * @implementation
 * a constant object, enforces range invariant at construction.
 */
class Float01
{
/// standard object services -----------------------------------------------------------------------
public:
	         Float01();
	//explicit Float01( float );

	        ~Float01();
	         Float01( const Float01& );
	Float01& operator=( const Float01& );


/// implicit conversions
	         Float01( float );
	         operator float() const;

private:
	         Float01( bool, float );


/// commands ---------------------------------------------------------------------------------------
public:
	        //void  set( float );


/// queries ----------------------------------------------------------------------------------------
public:
	        //float get() const;

	static  float clamp01( float );
	static  float moveUp( const Float01& );   /// change to (0,1] interval

	static  Float01 MIN();
	static  Float01 MAX();
	static  Float01 ZERO();
	static  Float01 HALF();
	static  Float01 ALMOST_ONE();


/// fields -----------------------------------------------------------------------------------------
private:
	float f01_m;
};








/// standard object services -----------------------------------------------------------------------
inline
Float01::Float01()
 :	f01_m( 0.0f )
{
}


inline
Float01::Float01( const float f )
 :	f01_m( Float01::clamp01( f ) )
{
}


inline
Float01::Float01( bool, const float f )
 :	f01_m( f )
{
}


inline
Float01::~Float01()
{
}


inline
Float01::Float01( const Float01& other )
 :	f01_m( other.f01_m )
{
}


inline
Float01& Float01::operator=( const Float01& other )
{
	f01_m = other.f01_m;

	return *this;
}


inline
Float01::operator float() const
{
	return f01_m;
}




///// commands ---------------------------------------------------------------------------------------
//inline
//void Float01::set( const float f )
//{
//	f01_m = clamp01( f );
//}
//
//
//
//
///// queries ----------------------------------------------------------------------------------------
//inline
//float Float01::get() const
//{
//	return f01_m;
//}




inline
float Float01::moveUp( const Float01& f )
{
	return f.f01_m + FLOAT_EPSILON;
}




inline
Float01 Float01::MIN()
{
	return Float01(false, 0.0f);
}


inline
Float01 Float01::MAX()
{
	return Float01(false, FLOAT_ALMOST_ONE);
}


inline
Float01 Float01::ZERO()
{
	return Float01(false, 0.0f);
}


inline
Float01 Float01::HALF()
{
	return Float01(false, 0.5f);
}


inline
Float01 Float01::ALMOST_ONE()
{
	return Float01(false, FLOAT_ALMOST_ONE);
}


}}




#endif//Float01_h
