/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Float01c_h
#define Float01c_h


#include "Primitives.h"




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a fp value between 0 and 1, ie: [0,1].<br/><br/>
 *
 * provides implicit conversion to and from float.
 *
 * @implementation
 * a constant object, enforces range invariant at construction.
 */
class Float01c
{
/// standard object services -----------------------------------------------------------------------
public:
	         Float01c();
	//explicit Float01c( float );

	        ~Float01c();
	         Float01c( const Float01c& );
	Float01c& operator=( const Float01c& );


/// implicit conversions
	         Float01c( float );
	         operator float() const;

private:
	         Float01c( bool, float );


/// commands ---------------------------------------------------------------------------------------
public:
	        //void  set( float );


/// queries ----------------------------------------------------------------------------------------
public:
	        //float get() const;

	static  float clamp01( float );

	static  Float01c MIN();
	static  Float01c MAX();
	static  Float01c ZERO();
	static  Float01c HALF();
	static  Float01c ONE();


/// fields -----------------------------------------------------------------------------------------
private:
	float f01_m;
};








/// standard object services -----------------------------------------------------------------------
inline
Float01c::Float01c()
 :	f01_m( 0.0f )
{
}


inline
Float01c::Float01c( const float f )
 :	f01_m( Float01c::clamp01( f ) )
{
}


inline
Float01c::Float01c( bool, const float f )
 :	f01_m( f )
{
}


inline
Float01c::~Float01c()
{
}


inline
Float01c::Float01c( const Float01c& other )
 :	f01_m( other.f01_m )
{
}


inline
Float01c& Float01c::operator=( const Float01c& other )
{
	f01_m = other.f01_m;

	return *this;
}


inline
Float01c::operator float() const
{
	return f01_m;
}




///// commands ---------------------------------------------------------------------------------------
//inline
//void Float01c::set( const float f )
//{
//	f01_m = clamp01( f );
//}
//
//
//
//
///// queries ----------------------------------------------------------------------------------------
//inline
//float Float01c::get() const
//{
//	return f01_m;
//}




inline
Float01c Float01c::MIN()
{
	return Float01c(false, 0.0f);
}


inline
Float01c Float01c::MAX()
{
	return Float01c(false, 1.0f);
}


inline
Float01c Float01c::ZERO()
{
	return Float01c(false, 0.0f);
}


inline
Float01c Float01c::HALF()
{
	return Float01c(false, 0.5f);
}


inline
Float01c Float01c::ONE()
{
	return Float01c(false, 1.0f);
}


}}




#endif//Float01c_h
