/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Float16_h
#define Float16_h


#include "Primitives.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a 16bit fp value: ieee standard, but with 8bit mantissa.<br/><br/>
 *
 * provides explicit conversion to and from float.
 */
class Float16
{
/// standard object services -----------------------------------------------------------------------
public:
	         Float16();
	explicit Float16( float );

	        ~Float16();
	         Float16( const Float16& );
	Float16& operator=( const Float16& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( float );


/// queries ----------------------------------------------------------------------------------------
public:
	        float get();

	static  word  fromFloat( float );
	static  float toFloat( word );


/// fields -----------------------------------------------------------------------------------------
private:
	/// 1     8       8
	/// 0 00000000 0000000 : sign, exponent, mantissa
	/// mean error: 0.17% ish
	word fp16_m;
};








/// INLINES ///


/// standard object services -----------------------------------------------------------------------
inline
Float16::Float16()
 :	fp16_m( 0 )
{
}


inline
Float16::Float16( const float fp )
 :	fp16_m( Float16::fromFloat( fp ) )
{
}


inline
Float16::~Float16()
{
}


inline
Float16::Float16( const Float16& other )
 :	fp16_m( other.fp16_m )
{
}


inline
Float16& Float16::operator=( const Float16& other )
{
	fp16_m = other.fp16_m;

	return *this;
}




/// commands ---------------------------------------------------------------------------------------
inline
void Float16::set( const float fp )
{
	fp16_m = fromFloat( fp );
}


/// queries ----------------------------------------------------------------------------------------
inline
float Float16::get()
{
	return toFloat( fp16_m );
}


}}




#endif//Float16_h
