/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef HaltonSequence_h
#define HaltonSequence_h


#include "Primitives.h"
#include "Sequence.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a halton sequence.<br/><br/>
 *
 * only the first 15 primes are implemented.<br/><br/>
 *
 * allows random access to the sequence.<br/><br/>
 *
 * jittering is inversely proportional to index.
 *
 * @implementation
 * adapted from:
 * <cite>'Sampling with Hammersley and Halton Points' - Wong, Luk, Heng
 * (Journal of Graphics Tools, vol2, no2, 1997)</cite>
 */
class HaltonSequence
	: public Sequence
{
/// standard object services -----------------------------------------------------------------------
public:
	         HaltonSequence();
	         /**
	          * @param base the index of the prime: first index is 0,
	          * corresponding to first prime, 2.
	          */
	explicit HaltonSequence( dword base );

	        ~HaltonSequence();
	         HaltonSequence( const HaltonSequence& );
	HaltonSequence& operator=( const HaltonSequence& );


/// commands ---------------------------------------------------------------------------------------
	        /**
	         * @param base the index of the prime: first index is 0,
	         * corresponding to first prime, 2.
	         */
	        void  setBase( dword base );
	        /** @param index value >= 0 */
	        void  setIndex( dword index );

	        void  next();


/// queries ----------------------------------------------------------------------------------------
	        dword getPrime()        const;
	        dword getIndex()        const;

	        float getItem()         const;


	        /** @param index value >= 0 */
	static  float getItem( dword prime, dword index );

	        /** @param index value >= 0 */
	static  float getItemJittered( dword prime, dword index, float rand01 );
	        /** @param index value >= 0 */
	static  float jitterPoint( float point, dword index, float rand01 );


/// fields -----------------------------------------------------------------------------------------
private:
	dword prime_m;

	dword index_m;
	float point_m;

	static const dword PRIMES[];
};








/// inlines ///

/// queries ----------------------------------------------------------------------------------------
inline
dword HaltonSequence::getPrime() const
{
	return prime_m;
}


inline
dword HaltonSequence::getIndex() const
{
	return index_m;
}


inline
float HaltonSequence::getItem() const
{
	return point_m;
}


}}




#endif//HaltonSequence_h
