/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef HistogramSmall_h
#define HistogramSmall_h


#include "Primitives.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


/**
 * a histogram for stochastic element choosing.<br/><br/>
 *
 * elements sum to 1, small number of elements, stored on the stack.<br/><br/>
 *
 * a collection of float elements, each representing a weighting, is built. then,
 * by giving a uniform random, selection of an item can be made with probability
 * in proportion to the items weight.<br/><br/>
 *
 * @invariants
 * length_m <= MAX_LENGTH<br/>
 * length_m >= 1<br/>
 * widths_m elements are each >= 0.0<br/>
 * widths_m elements sum to 1.0f<br/>
 */
class HistogramSmall
{
/// standard object services -----------------------------------------------------------------------
public:
	         HistogramSmall();
	explicit HistogramSmall( dword length );
	         HistogramSmall( dword       length,
	                         const float values[] );

	        ~HistogramSmall();
	         HistogramSmall( const HistogramSmall& );
	HistogramSmall& operator=( const HistogramSmall& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setElements( dword       length,
	                           const float values[] );

	static  bool  test( bool isLogging );


/// queries ----------------------------------------------------------------------------------------
public:
	        dword getChoice( Float01  stochastic )                                            const;
	        dword getChoice( Float01  stochastic,
	                         Float01& stochasticRescaled )                                    const;

	        const float* getElements()                                                        const;
	        dword        getLength()                                                          const;

	static  bool isChosen( Float01  proportion,
	                       Float01  stochastic,
	                       Float01& stochasticRescaled );

	static  dword getMaxLength();


/// fields -----------------------------------------------------------------------------------------
private:
	static const dword MAX_LENGTH = 8;

	dword length_m;
	float widths_m[MAX_LENGTH];
};








/// queries ----------------------------------------------------------------------------------------
inline
const float* HistogramSmall::getElements() const
{
	return widths_m;
}


inline
dword HistogramSmall::getLength() const
{
	return length_m;
}


inline
dword HistogramSmall::getMaxLength()
{
	return MAX_LENGTH;
}


}}




#endif//HistogramSmall_h
