/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef LogThreadSafe_h
#define LogThreadSafe_h


#include "Primitives.h"
#include "ThreadLock.h"
#include <string>
#include <fstream>
#include <memory>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::string;
	using std::ofstream;
	using std::auto_ptr;


/**
 * a global log singleton.<br/><br/>
 *
 * init must be called first.<br/><br/>
 *
 * writes a '\n', then the thread id, then the string, (then flushes) eg:<br/>
 * <code>~0000017C~  rendered frame 1 at ...</code>
 */
class LogThreadSafe
{
/// standard object services -----------------------------------------------------------------------
private:
	        LogThreadSafe( const char* filePathname, bool enable );
public:
	       ~LogThreadSafe();
private:
	        LogThreadSafe( const LogThreadSafe& );
	LogThreadSafe& operator=( const LogThreadSafe& );


/// commands ---------------------------------------------------------------------------------------
public:
	static  void  init( const char* filePathname, bool enable );   /// must be called first

	static  void  enable( bool );
	static  void  write( const string& );


/// queries ----------------------------------------------------------------------------------------
public:
	static  bool  isEnabled();


/// implementation ---------------------------------------------------------------------------------
private:
	static  LogThreadSafe& get();

	        void  enable_( bool );
	        void  write_( const string& );

	        bool  isEnabled_() const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// static
	static const char FILE_NAME_DEFAULT[];

	static string filePathname_s;
	static bool   enabled_s;

	static auto_ptr<ThreadLock> pThreadLock_s;


	/// instance
	string        filePathname_m;
	bool          enabled_m;

	ofstream      logFile_m;
};








/// INLINES ///


inline
bool LogThreadSafe::isEnabled_() const
{
	return enabled_m;
}




}}




#endif//LogThreadSafe_h
