/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "SerializerOut.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char SerializerOut::CLASS_NAME[]        = "hxa7241::utility::SerializerOut";
const char SerializerOut::EXCEPTION_MESSAGE[] = "serialization stream failure";




/// standard object services -----------------------------------------------------------------------
SerializerOut::SerializerOut
(
	ostream& out
)
 :	out_m( out ),
	buffer_m( 128, '\0' ),
	previousExceptionFlags_m()
{
	const char METHOD_NAME[] = "constructor";

	try
	{
		previousExceptionFlags_m = out_m.exceptions();
		out_m.exceptions( ostream::badbit | ostream::failbit | ostream::eofbit );
	}
	catch( std::ios_base::failure e )
	{
		throw hxa7241::utility::Exception( EXCEPTION_MESSAGE, CLASS_NAME, METHOD_NAME );
	}
}


SerializerOut::~SerializerOut()
{
	out_m.exceptions( previousExceptionFlags_m );
}




/// implementation ---------------------------------------------------------------------------------
SerializerOut& SerializerOut::operator<<( const void* a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const bool a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const hxa7241::utility::byte a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const ubyte a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const word a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const uword a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const dword a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const udword a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const long a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const unsigned long a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const float a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const double a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const char a )
{
	out_m << a;

	return *this;
}


SerializerOut& SerializerOut::operator<<( const char* cp )
{
	if( cp != 0 )
	{
		out_m << cp << '\0';
	}

	return *this;
}


SerializerOut& SerializerOut::operator<<( const string& s )
{
	out_m << s.c_str() << '\0';

	return *this;
}




/// implementation ---------------------------------------------------------------------------------
void SerializerOut::writeHeader
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	/// default do nothing
}


void SerializerOut::writeFooter
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	/// default do nothing
}


void SerializerOut::writePreObject
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	/// default do nothing
}


void SerializerOut::writePostObject
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	/// default do nothing
}




const char* SerializerOut::COLLECTION_ELEMENT_NAME()
{
	static const char COLLECTION_ELEMENT_NAME[] = "element";
	return COLLECTION_ELEMENT_NAME;
}


const char* SerializerOut::COLLECTION_SIZE_NAME()
{
	static const char COLLECTION_SIZE_NAME[] = "elementCount";
	return COLLECTION_SIZE_NAME;
}
