/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ThreadLock_h
#define ThreadLock_h


#include "Primitives.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	class ThreadSync;


/**
 * abstract base of a thread synchronisation class.<br/><br/>
 *
 * derivatives must implement lock and unlock to support thread synchonisation,
 * unlock undoing whatever lock does.
 *
 * @see ThreadSync
 */
class ThreadLock
{
/// standard object services -----------------------------------------------------------------------
protected:
	        ThreadLock() {}
public:
	       ~ThreadLock() {}
private:
	        ThreadLock( const ThreadLock& );
	ThreadLock& operator=( const ThreadLock& );


/// commands ---------------------------------------------------------------------------------------
private:
	friend class ThreadSync;

	virtual void lock()   =0;
	virtual void unlock() =0;
};




/**
 * ties thread locking to an object lifetime.<br/><br/>
 *
 * allows simple one code line temporary thread locking, since unlocking is
 * automatically induced by destruction.
 *
 * @see ThreadLock
 */
class ThreadSync
{
/// standard object services -----------------------------------------------------------------------
public:
	        ThreadSync( ThreadLock* pLock )
	         :	pLock_m( pLock )
	        {
	        	pLock_m->lock();
	        }

	       ~ThreadSync()
	        {
	        	pLock_m->unlock();
	        }
private:
	        ThreadSync( const ThreadSync& );
	ThreadSync& operator=( const ThreadSync& );


/// fields -----------------------------------------------------------------------------------------
private:
	ThreadLock* pLock_m;
};


}}




#endif//ThreadLock_h
