/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include <time>
#include <sstream>

#include "TimePoint.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char TimePoint::RFC1123_FORMAT[] = "%a, %d %b %Y %H:%M:%S GMT";   /// Wed, 09 Feb 2002 19:37:02 GMT
const char TimePoint::ISO8601_FORMAT[] = "%Y-%m-%dT%H:%M:%SZ";          /// 2002-02-09T19:37:02Z
const char TimePoint::TIME_FORMAT[]    = "%H:%M:%S";                    /// 19:37:02
const char TimePoint::DATE_FORMAT[]    = "%Y-%m-%d";                    /// 2002-02-09




/// standard object services -----------------------------------------------------------------------
TimePoint::TimePoint()
 :	seconds_m( dword(std::time(0)) )
{
}


TimePoint::TimePoint( const dword seconds )
 :	seconds_m( seconds )
{
}


TimePoint::~TimePoint()
{
}


TimePoint::TimePoint( const TimePoint& other )
 :	seconds_m( other.seconds_m )
{
}


TimePoint& TimePoint::operator=( const TimePoint& other )
{
	seconds_m = other.seconds_m;

	return *this;
}




/// queries ----------------------------------------------------------------------------------------
dword TimePoint::getSeconds() const
{
	return seconds_m;
}


string TimePoint::getStringRFC1123() const
{
	string s;
	format( RFC1123_FORMAT, seconds_m, s );

	return s;
}


string TimePoint::getStringISO8601() const
{
	string s;
	format( ISO8601_FORMAT, seconds_m, s );

	return s;
}


string TimePoint::getStringTime() const
{
	string s;
	format( TIME_FORMAT, seconds_m, s );

	return s;
}


string TimePoint::getStringDate() const
{
	string s;
	format( DATE_FORMAT, seconds_m, s );

	return s;
}




/// statics
string TimePoint::getStringRFC1123( const dword seconds )
{
	string s;
	format( RFC1123_FORMAT, seconds, s );

	return s;
}


string TimePoint::getStringISO8601( const dword seconds )
{
	string s;
	format( ISO8601_FORMAT, seconds, s );

	return s;
}


string TimePoint::getStringTime( const dword seconds )
{
	string s;
	format( TIME_FORMAT, seconds, s );

	return s;
}


string TimePoint::getStringDate( const dword seconds )
{
	string s;
	format( DATE_FORMAT, seconds, s );

	return s;
}


string TimePoint::getStringDuration( const dword seconds )
{
	const dword secs    = seconds % 60;
	const dword minutes = seconds / 60;
	const dword mins    = minutes % 60;
	const dword hours   = minutes / 60;

	std::ostringstream oss;

	oss << hours << "h " << mins << "m " << secs << "s";

	return string( oss.str() );
}




/// implementation ---------------------------------------------------------------------------------
void TimePoint::format( const char* format, const dword seconds, string& s )
{
	const time_t     t   = time_t(seconds);
	const struct tm* pTm = ::gmtime( &t );

	char buf[32];
	std::strftime( buf, (sizeof(buf)/sizeof(*buf)), format, pTm );

	s = buf;
}
