/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef TimePoint_h
#define TimePoint_h


#include "Primitives.h"
#include <string>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::string;


/**
 * a point in time, in seconds, with two standard text formatters.<br/><br/>
 *
 * the measurement is the (whole) seconds since 1970 beginning<br/><br/>
 *
 * RFC1123 string format is the one used in HTTP 1.1, eg:<br/>
 * 	  Wed, 09 Feb 2002 19:37:02 GMT<br/><br/>
 *
 * ISO8601 string format is a conventional simple 'profile' of what is a family
 * of formats, eg:<br/>
 * 	  2002-02-09T19:37:02Z<br/><br/>
 *
 * time is the time portion of ISO8601<br/>
 * date is the date portion of ISO8601<br/><br/>
 *
 * reference: <cite>http://www.w3.org/TR/NOTE-datetime</cite>
 */
class TimePoint
{
/// standard object services -----------------------------------------------------------------------
public:
	         TimePoint();
	explicit TimePoint( dword seconds );

	        ~TimePoint();
	         TimePoint( const TimePoint& );
	TimePoint& operator=( const TimePoint& );


/// commands ---------------------------------------------------------------------------------------
public:


/// queries ----------------------------------------------------------------------------------------
public:
	         dword  getSeconds()       const;

	         string getStringRFC1123() const;	/// Wed, 09 Feb 2002 19:37:02 GMT
	         string getStringISO8601() const;	/// 2002-02-09T19:37:02Z
	         string getStringTime()    const;	/// 19:37:02
	         string getStringDate()    const;	/// 2002-02-09

	static   string getStringRFC1123( dword seconds );
	static   string getStringISO8601( dword seconds );
	static   string getStringTime   ( dword seconds );
	static   string getStringDate   ( dword seconds );

	static   string getStringDuration( dword seconds );   /// 47h 12m 35s


/// implementation ---------------------------------------------------------------------------------
protected:
	static   void   format( const char* strftimeFormat, dword seconds, string& );


/// fields -----------------------------------------------------------------------------------------
private:
	dword seconds_m;

	static const char RFC1123_FORMAT[];
	static const char ISO8601_FORMAT[];
	static const char TIME_FORMAT[];
	static const char DATE_FORMAT[];
};


}}




#endif//TimePoint_h
