/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef XmlSerializerIn_h
#define XmlSerializerIn_h


#include "Primitives.h"
#include "SerializerIn.h"
#include <fstream>
#include <string>




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::istream;
	using std::string;


/**
 * a xml file inward serializer.<br/><br/>
 *
 * the object in the xml file must match the receiving object type exactly.
 * <br/><br/>
 *
 * the outward serializer of the xml file must match this inward serializer
 * version.
 *
 * @see XmlSerializerOut
 */
class XmlSerializerIn
	: public SerializerIn
{
/// standard object services -----------------------------------------------------------------------
private:
	         XmlSerializerIn( istream&, const char* fileName );

	        ~XmlSerializerIn();
	         XmlSerializerIn( const XmlSerializerIn& );
	XmlSerializerIn& operator=( const XmlSerializerIn& );


/// commands ---------------------------------------------------------------------------------------
public:
	template<class TYPE>
	static  bool  serializeIn( TYPE& object, const char* filePathname );
	template<class TYPE>
	static  bool  serializeIn( TYPE& object, istream&, const char* fileName );


/// queries ----------------------------------------------------------------------------------------
public:


/// implementation ---------------------------------------------------------------------------------
private:
	/// overrides
	virtual XmlSerializerIn& operator>>( void*& );
	virtual XmlSerializerIn& operator>>( bool& );
	virtual XmlSerializerIn& operator>>( hxa7241::utility::byte& );
	virtual XmlSerializerIn& operator>>( ubyte& );
	virtual XmlSerializerIn& operator>>( word& );
	virtual XmlSerializerIn& operator>>( uword& );
	virtual XmlSerializerIn& operator>>( dword& );
	virtual XmlSerializerIn& operator>>( udword& );
	virtual XmlSerializerIn& operator>>( long& );
	virtual XmlSerializerIn& operator>>( unsigned long& );
	virtual XmlSerializerIn& operator>>( float& );
	virtual XmlSerializerIn& operator>>( double& );
	virtual XmlSerializerIn& operator>>( char& );
	virtual XmlSerializerIn& operator>>( char* );
	virtual XmlSerializerIn& operator>>( string& );

	virtual void  readHeader( const char* name );
//	virtual void  readFooter( const char* name );

	virtual bool  readPreObject ( const char* name );
	virtual void  readPostObject( const char* name );
	virtual bool  readPreElement( const char* collectionName, const char* name, bool& isCollectionEnd );

	/// other
	struct Node
	{
		enum ENodeType { NOTUSED, PI, OPEN, CLOSE, EMPTY };

		ENodeType type_m;
		string    content_m;
	};

	        void  readProlog  ( const char* className );
	        bool  readTagOpen ( const char* name );
	        void  readTagClose( const char* name );
	        bool  readTagOpenElement( const char* collectionName, const char* name, bool& isCollectionEnd );
	        void  readString( string& s );
	        void  readChars( char* );

	        void  readNode();
	        void  readMarkup();
	        void  readUntilPair  ( const char* pair,   string& );
	        void  readUntilTriple( const char* triple, string* );
	        char  readChar();
	        void  checkCdataOpen();
	        void  throwException( const char* pMessage, const char* pMethodName );

	static  bool  isBlank( char );
	static  bool  isBlank( const string& );


/// fields -----------------------------------------------------------------------------------------
private:
	Node     nodeBuffer_m;

	dword    lineNumber_m;
	dword    charPos_m;
	bool     lastCharWasNewline_m;


	static const char EXCEPTION_MESSAGE1[];
	static const char EXCEPTION_MESSAGE2[];
	static const char EXCEPTION_MESSAGE3[];
	static const char EXCEPTION_MESSAGE4[];
	static const char CLASS_NAME[];
};








/// TEMPLATES ///


/// commands ---------------------------------------------------------------------------------------
/// static
template<class TYPE>
bool XmlSerializerIn::serializeIn
(
	TYPE&            object,
	const char*const filePathname
)
{
	std::ifstream file( filePathname );

	return XmlSerializerIn::serializeIn( object, file, filePathname );
}


template<class TYPE>
bool XmlSerializerIn::serializeIn
(
	TYPE&            object,
	istream&         in,
	const char*const fileName
)
{
	XmlSerializerIn xsi( in, fileName );

	return xsi.read( object );
}


}}




#endif//XmlSerializerIn_h
