/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include <cctype>

#include "XmlSerializerOut.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char XmlSerializerOut::CLASS_NAME[]             = "hxa7241::utility::XmlSerializerOut";

const char XmlSerializerOut::XML_DECLARATION[]        = "xml version='1.0' encoding='UTF-8' standalone='yes'";
const char XmlSerializerOut::SERIALIZER_DECLARATION[] = "hxa7241-XmlSerializer version='1.0'";
const char XmlSerializerOut::SERIALIZER_COMMENTS[]    = "'object serialized to xml' 'exact class match required'";
const char XmlSerializerOut::NEWLINE[]                = "\n";//"\r\n";
const char XmlSerializerOut::SPACE[]                  = " ";
const char XmlSerializerOut::INDENT[]                 = "\t";
const char XmlSerializerOut::DEFAULT_NAME[]           = "anonymous";
const char XmlSerializerOut::PI_OPEN[]                = "<?";
const char XmlSerializerOut::PI_CLOSE[]               = "?>";
const char XmlSerializerOut::CDATA_OPEN[]             = "<![CDATA[";
const char XmlSerializerOut::CDATA_CLOSE[]            = "]]>";
const char XmlSerializerOut::COMMENT_OPEN[]           = "<!-- ";
const char XmlSerializerOut::COMMENT_CLOSE[]          = " -->";
//const char XmlSerializerOut::ARRAY_ELEMENT_NAME[] = "element";




/// standard object services -----------------------------------------------------------------------
XmlSerializerOut::XmlSerializerOut
(
	ostream& out
)
 :	SerializerOut( out ),
	indent_m( 0 ),
	wasPrimitive_m( false )
{
}


XmlSerializerOut::~XmlSerializerOut()
{
}




/// implementation ---------------------------------------------------------------------------------
XmlSerializerOut& XmlSerializerOut::operator<<( const void* a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const bool a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const hxa7241::utility::byte a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const ubyte a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const word a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const uword a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const dword a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const udword a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const long a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const unsigned long a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const float a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const double a )
{
	out_m << a;
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const char a )
{
	const char cp[] = { a, '\0' };
	writeString( cp );
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const char* cp )
{
	writeString( cp );
	wasPrimitive_m = true;

	return *this;
}


XmlSerializerOut& XmlSerializerOut::operator<<( const string& s )
{
	writeString( s.c_str() );
	wasPrimitive_m = true;

	return *this;
}




/// implementation ---------------------------------------------------------------------------------
void XmlSerializerOut::writeHeader
(
	const char* name,
	const char* ,//typeName,
	const char* //comment
)
{
	writeProlog( name );
}


void XmlSerializerOut::writeFooter
(
	const char* ,//name,
	const char* ,//typeName,
	const char* //comment
)
{
	out_m << NEWLINE;
}


void XmlSerializerOut::writePreObject
(
	const char* name,
	const char* typeName,
	const char* comment
)
{
	writeComment( comment );
	writeTagOpen( name, typeName );
}


void XmlSerializerOut::writePostObject
(
	const char* name,
	const char* ,//typeName,
	const char* //comment
)
{
	writeTagClose( name );
}




void XmlSerializerOut::writeProlog
(
	const char*const //name
) const
{
	out_m << PI_OPEN << XML_DECLARATION << PI_CLOSE << NEWLINE;
	out_m << PI_OPEN << SERIALIZER_DECLARATION << SPACE << SERIALIZER_COMMENTS << PI_CLOSE << NEWLINE;
}


void XmlSerializerOut::writeComment
(
	const char*const comment
) const
{
	if( comment != 0 )
	{
		/// write formatting
		out_m << NEWLINE;
		writeIndent( indent_m );

		/// start commment
		out_m << COMMENT_OPEN;
		{
			bool lastWasHyphen = false;
			for( const char* pc = comment;  *pc != 0;  ++pc )
			{
				char c = *pc;
				/// check for embedded "--"
				if( lastWasHyphen & (c == '-') )
				{
					c = '_';
				}
				/// write char
				out_m << c;
				lastWasHyphen = (c == '-');
			}
		}
		/// end comment
		out_m << COMMENT_CLOSE;
	}
}


void XmlSerializerOut::writeTagOpen
(
	const char*const name,
	const char*const typeName
)
{
	/// write formatting
	out_m << NEWLINE;
	writeIndent( indent_m );

	/// write start and name
	out_m << '<';
	maskName( name, buffer_m );
	out_m << buffer_m.c_str();

	/// write type attribute
	maskAttributeValue( typeName, buffer_m );
	out_m << " type='" << buffer_m.c_str() << "'";
	out_m << '>';

	/// increase indent
	++indent_m;
}


void XmlSerializerOut::writeTagClose
(
	const char*const name
)
{
	/// decrease indent
	--indent_m;

	/// write formatting (if last content wasnt primitive type)
	if( !wasPrimitive_m )
	{
		out_m << NEWLINE;
		writeIndent( indent_m );
	}
	wasPrimitive_m = false;

	/// write tag
	out_m << "</";
	maskName( name, buffer_m );
	out_m << buffer_m.c_str();
	out_m << '>';
}


void XmlSerializerOut::writeIndent
(
	const dword amount
) const
{
	for( udword i = amount;  i-- > 0; )
	{
		out_m << INDENT;
	}
}


void XmlSerializerOut::writeString
(
	const char* pContent
) const
{
	if( pContent != 0 )
	{
		/// write CDATA section start
		out_m << CDATA_OPEN;

		char c[3] = { ' ', ' ', ' ' };
		while( *pContent != 0 )
		{
			c[0] = c[1];
			c[1] = c[2];
			c[2] = *pContent++;
			/// check for embedded "]]>"
			if( (c[0] == CDATA_CLOSE[0]) & (c[1] == CDATA_CLOSE[1]) & (c[2] == CDATA_CLOSE[2]) )
			{
				/// break it up with an inserted space
				out_m << ' ';
			}

			/// write char
			out_m << c[2];
		}

		/// write CDATA section end
		out_m << CDATA_CLOSE;

//		while( *pContent != 0 )
//		{
//			char c = *pContent++;
//			switch( c )
//			{
//			case '<'  :  out_m << "&lt;";    break;
//			case '>'  :  out_m << "&gt;";    break;
//			case '&'  :  out_m << "&amp;";   break;
//			case '\'' :  out_m << "&apos;";  break;
//			case '\"' :  out_m << "&quot;";  break;
//			default   :  out_m << c;         break;
//			}
//		}
	}
}


void XmlSerializerOut::maskName
(
	const char* pName,
	string&     name
)
{
	/// check for empty name
	if( (pName == 0) | (*pName == 0) )
	{
		pName = DEFAULT_NAME;
	}

	/// clear result
	name = "";

	/// check first char
	char c = *pName++;
	if( (c != '_') && !bool(std::isalpha( int(c) )) )
	{
		c =  '_';
	}
	name += c;

	/// check other chars
	while( *pName != 0 )
	{
		c = *pName++;
		if( ((c != '.') & (c != '-') & (c != '_')) && !bool(std::isalnum( int(c) )) )
		{
			c =  '_';
		}
		/// append char
		name += c;
	}
}


void XmlSerializerOut::maskAttributeValue
(
	const char* pValue,
	string&     value
)
{
	if( (pValue != 0) )
	{
		/// clear result
		value = "";

		char c;
		while( (c = *pValue++) != 0 )
		{
			/// check for illegal chars
			if( (c == '^') | (c == '<') | (c == '&') | (c == '"') | (c == '\'') )
			{
				c =  '_';
			}
			/// append char
			value += c;
		}
	}
}
