/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef XmlSerializerOut_h
#define XmlSerializerOut_h


#include "Primitives.h"
#include "SerializerOut.h"
#include <fstream>
#include <string>




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::ostream;
	using std::string;


/**
 * a xml file outward serializer.<br/><br/>
 *
 * writes an xml file reflecting the structure of the object.
 *
 * @see XmlSerializerIn
 */
class XmlSerializerOut
	: public SerializerOut
{
/// standard object services -----------------------------------------------------------------------
protected:
	explicit XmlSerializerOut( ostream& );

	virtual ~XmlSerializerOut();
private:
	         XmlSerializerOut( const XmlSerializerOut& );
	XmlSerializerOut& operator=( const XmlSerializerOut& );


/// commands ---------------------------------------------------------------------------------------
public:
	template<class TYPE>
	static  void  serializeOut( const TYPE& object, const char* filePathname );
	template<class TYPE>
	static  void  serializeOut( const TYPE& object, ostream& );


/// queries ----------------------------------------------------------------------------------------
public:


/// implementation ---------------------------------------------------------------------------------
protected:
	/// overrides
	virtual XmlSerializerOut& operator<<( const void* );
	virtual XmlSerializerOut& operator<<( bool );
	virtual XmlSerializerOut& operator<<( hxa7241::utility::byte );
	virtual XmlSerializerOut& operator<<( ubyte );
	virtual XmlSerializerOut& operator<<( word );
	virtual XmlSerializerOut& operator<<( uword );
	virtual XmlSerializerOut& operator<<( dword );
	virtual XmlSerializerOut& operator<<( udword );
	virtual XmlSerializerOut& operator<<( long );
	virtual XmlSerializerOut& operator<<( unsigned long );
	virtual XmlSerializerOut& operator<<( float );
	virtual XmlSerializerOut& operator<<( double );
	virtual XmlSerializerOut& operator<<( char );
	virtual XmlSerializerOut& operator<<( const char* );
	virtual XmlSerializerOut& operator<<( const string& );

	virtual void  writeHeader( const char* name, const char* typeName, const char* comment );
	virtual void  writeFooter( const char* name, const char* typeName, const char* comment );

	virtual void  writePreObject ( const char* name, const char* typeName, const char* comment );
	virtual void  writePostObject( const char* name, const char* typeName, const char* comment );

	/// other
	virtual void  writeProlog  ( const char* className )                   const;
	virtual void  writeComment ( const char* comment )                     const;
	virtual void  writeIndent  ( dword amount )                            const;
	virtual void  writeTagOpen ( const char* name, const char* typeName );
	virtual void  writeTagClose( const char* name );
	virtual void  writeString  ( const char* content )                     const;

public:
	static  void  maskName( const char*, string& );
	static  void  maskAttributeValue( const char*, string& );


/// fields -----------------------------------------------------------------------------------------
protected:
	dword indent_m;
	bool  wasPrimitive_m;

public:
	static const char CLASS_NAME[];

	static const char XML_DECLARATION[];
	static const char SERIALIZER_DECLARATION[];
	static const char SERIALIZER_COMMENTS[];
	static const char NEWLINE[];
	static const char SPACE[];
	static const char INDENT[];
	static const char DEFAULT_NAME[];
	static const char PI_OPEN[];
	static const char PI_CLOSE[];
	static const char CDATA_OPEN[];
	static const char CDATA_CLOSE[];
	static const char COMMENT_OPEN[];
	static const char COMMENT_CLOSE[];
//	static const char ARRAY_ELEMENT_NAME[];
};








/// TEMPLATES ///


/// commands ---------------------------------------------------------------------------------------
/// static
template<class TYPE>
void XmlSerializerOut::serializeOut
(
	const TYPE&      object,
	const char*const filePathname
)
{
	std::ofstream file( filePathname, std::ofstream::trunc );

	XmlSerializerOut::serializeOut( object, file );
}


template<class TYPE>
void XmlSerializerOut::serializeOut
(
	const TYPE& object,
	ostream&    out
)
{
	XmlSerializerOut xso( out );

	xso.write( object );
}


}}




#endif//XmlSerializerOut_h
